"""Generated CLI commands for Libraries tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""An external library is made up of input file paths or expressions that are scanned for asset files. Discovered files are automatically imported. Assets much be unique within a library, but can be duplicated across libraries. Each user has a default upload library, and can have one or more external libraries.\n\n[link=https://api.immich.app/endpoints/libraries]Immich API documentation[/link]"""
)


@app.command("create-library", deprecated=False, rich_help_panel="API commands")
def create_library(
    ctx: typer.Context,
    exclusion_patterns: list[str] | None = typer.Option(
        None, "--exclusion-patterns", help=""""""
    ),
    import_paths: list[str] | None = typer.Option(None, "--import-paths", help=""""""),
    name: str | None = typer.Option(None, "--name", help=""""""),
    owner_id: str = typer.Option(..., "--owner-id", help=""""""),
) -> None:
    """Create a library

    [link=https://api.immich.app/endpoints/libraries/createLibrary]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if exclusion_patterns is not None:
        set_nested(json_data, ["exclusion_patterns"], exclusion_patterns)
    if import_paths is not None:
        set_nested(json_data, ["import_paths"], import_paths)
    if name is not None:
        set_nested(json_data, ["name"], name)
    set_nested(json_data, ["owner_id"], owner_id)
    create_library_dto = CreateLibraryDto.model_validate(json_data)
    kwargs["create_library_dto"] = create_library_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.libraries, "create_library", ctx, **kwargs)
    print_response(result, ctx)


@app.command("delete-library", deprecated=False, rich_help_panel="API commands")
def delete_library(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Delete a library

    [link=https://api.immich.app/endpoints/libraries/deleteLibrary]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.libraries, "delete_library", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-all-libraries", deprecated=False, rich_help_panel="API commands")
def get_all_libraries(
    ctx: typer.Context,
) -> None:
    """Retrieve libraries

    [link=https://api.immich.app/endpoints/libraries/getAllLibraries]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.libraries, "get_all_libraries", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-library", deprecated=False, rich_help_panel="API commands")
def get_library(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Retrieve a library

    [link=https://api.immich.app/endpoints/libraries/getLibrary]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.libraries, "get_library", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-library-statistics", deprecated=False, rich_help_panel="API commands")
def get_library_statistics(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Retrieve library statistics

    [link=https://api.immich.app/endpoints/libraries/getLibraryStatistics]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.libraries, "get_library_statistics", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("scan-library", deprecated=False, rich_help_panel="API commands")
def scan_library(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Scan a library

    [link=https://api.immich.app/endpoints/libraries/scanLibrary]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.libraries, "scan_library", ctx, **kwargs)
    print_response(result, ctx)


@app.command("update-library", deprecated=False, rich_help_panel="API commands")
def update_library(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    exclusion_patterns: list[str] | None = typer.Option(
        None, "--exclusion-patterns", help=""""""
    ),
    import_paths: list[str] | None = typer.Option(None, "--import-paths", help=""""""),
    name: str | None = typer.Option(None, "--name", help=""""""),
) -> None:
    """Update a library

    [link=https://api.immich.app/endpoints/libraries/updateLibrary]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    if exclusion_patterns is not None:
        set_nested(json_data, ["exclusion_patterns"], exclusion_patterns)
    if import_paths is not None:
        set_nested(json_data, ["import_paths"], import_paths)
    if name is not None:
        set_nested(json_data, ["name"], name)
    update_library_dto = UpdateLibraryDto.model_validate(json_data)
    kwargs["update_library_dto"] = update_library_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.libraries, "update_library", ctx, **kwargs)
    print_response(result, ctx)


@app.command("validate", deprecated=False, rich_help_panel="API commands")
def validate(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    exclusion_patterns: list[str] | None = typer.Option(
        None, "--exclusion-patterns", help=""""""
    ),
    import_paths: list[str] | None = typer.Option(None, "--import-paths", help=""""""),
) -> None:
    """Validate library settings

    [link=https://api.immich.app/endpoints/libraries/validate]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    if exclusion_patterns is not None:
        set_nested(json_data, ["exclusion_patterns"], exclusion_patterns)
    if import_paths is not None:
        set_nested(json_data, ["import_paths"], import_paths)
    validate_library_dto = ValidateLibraryDto.model_validate(json_data)
    kwargs["validate_library_dto"] = validate_library_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.libraries, "validate", ctx, **kwargs)
    print_response(result, ctx)



"""Generated CLI commands for Shared links tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from datetime import datetime
from typing import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""A shared link is a public url that provides access to a specific album, asset, or collection of assets. A shared link can be protected with a password, include a specific slug, allow or disallow downloads, and optionally include an expiration date.\n\n[link=https://api.immich.app/endpoints/shared-links]Immich API documentation[/link]"""
)


@app.command("add-shared-link-assets", deprecated=False, rich_help_panel="API commands")
def add_shared_link_assets(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    asset_ids: list[str] = typer.Option(..., "--asset-ids", help=""""""),
    key: str | None = typer.Option(None, "--key", help=""""""),
    slug: str | None = typer.Option(None, "--slug", help=""""""),
) -> None:
    """Add assets to a shared link

    [link=https://api.immich.app/endpoints/shared-links/addSharedLinkAssets]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    if key is not None:
        kwargs["key"] = key
    if slug is not None:
        kwargs["slug"] = slug
    set_nested(json_data, ["asset_ids"], asset_ids)
    asset_ids_dto = AssetIdsDto.model_validate(json_data)
    kwargs["asset_ids_dto"] = asset_ids_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.shared_links, "add_shared_link_assets", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("create-shared-link", deprecated=False, rich_help_panel="API commands")
def create_shared_link(
    ctx: typer.Context,
    album_id: str | None = typer.Option(None, "--album-id", help=""""""),
    allow_download: Literal["true", "false"] | None = typer.Option(
        None, "--allow-download", help=""""""
    ),
    allow_upload: Literal["true", "false"] | None = typer.Option(
        None, "--allow-upload", help=""""""
    ),
    asset_ids: list[str] | None = typer.Option(None, "--asset-ids", help=""""""),
    description: str | None = typer.Option(None, "--description", help=""""""),
    expires_at: datetime | None = typer.Option(None, "--expires-at", help=""""""),
    password: str | None = typer.Option(None, "--password", help=""""""),
    show_metadata: Literal["true", "false"] | None = typer.Option(
        None, "--show-metadata", help=""""""
    ),
    slug: str | None = typer.Option(None, "--slug", help=""""""),
    type: str = typer.Option(..., "--type", help=""""""),
) -> None:
    """Create a shared link

    [link=https://api.immich.app/endpoints/shared-links/createSharedLink]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if album_id is not None:
        set_nested(json_data, ["album_id"], album_id)
    if allow_download is not None:
        set_nested(json_data, ["allow_download"], allow_download.lower() == "true")
    if allow_upload is not None:
        set_nested(json_data, ["allow_upload"], allow_upload.lower() == "true")
    if asset_ids is not None:
        set_nested(json_data, ["asset_ids"], asset_ids)
    if description is not None:
        set_nested(json_data, ["description"], description)
    if expires_at is not None:
        set_nested(json_data, ["expires_at"], expires_at)
    if password is not None:
        set_nested(json_data, ["password"], password)
    if show_metadata is not None:
        set_nested(json_data, ["show_metadata"], show_metadata.lower() == "true")
    if slug is not None:
        set_nested(json_data, ["slug"], slug)
    set_nested(json_data, ["type"], type)
    shared_link_create_dto = SharedLinkCreateDto.model_validate(json_data)
    kwargs["shared_link_create_dto"] = shared_link_create_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.shared_links, "create_shared_link", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("get-all-shared-links", deprecated=False, rich_help_panel="API commands")
def get_all_shared_links(
    ctx: typer.Context,
    album_id: str | None = typer.Option(None, "--album-id", help=""""""),
    id: str | None = typer.Option(None, "--id", help=""""""),
) -> None:
    """Retrieve all shared links

    [link=https://api.immich.app/endpoints/shared-links/getAllSharedLinks]Immich API documentation[/link]
    """
    kwargs = {}
    if album_id is not None:
        kwargs["album_id"] = album_id
    if id is not None:
        kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.shared_links, "get_all_shared_links", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("get-my-shared-link", deprecated=False, rich_help_panel="API commands")
def get_my_shared_link(
    ctx: typer.Context,
    key: str | None = typer.Option(None, "--key", help=""""""),
    password: str | None = typer.Option(None, "--password", help="""password"""),
    slug: str | None = typer.Option(None, "--slug", help=""""""),
    token: str | None = typer.Option(None, "--token", help=""""""),
) -> None:
    """Retrieve current shared link

    [link=https://api.immich.app/endpoints/shared-links/getMySharedLink]Immich API documentation[/link]
    """
    kwargs = {}
    if key is not None:
        kwargs["key"] = key
    if password is not None:
        kwargs["password"] = password
    if slug is not None:
        kwargs["slug"] = slug
    if token is not None:
        kwargs["token"] = token
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.shared_links, "get_my_shared_link", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("get-shared-link-by-id", deprecated=False, rich_help_panel="API commands")
def get_shared_link_by_id(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Retrieve a shared link

    [link=https://api.immich.app/endpoints/shared-links/getSharedLinkById]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.shared_links, "get_shared_link_by_id", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("remove-shared-link", deprecated=False, rich_help_panel="API commands")
def remove_shared_link(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Delete a shared link

    [link=https://api.immich.app/endpoints/shared-links/removeSharedLink]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.shared_links, "remove_shared_link", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command(
    "remove-shared-link-assets", deprecated=False, rich_help_panel="API commands"
)
def remove_shared_link_assets(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    asset_ids: list[str] = typer.Option(..., "--asset-ids", help=""""""),
    key: str | None = typer.Option(None, "--key", help=""""""),
    slug: str | None = typer.Option(None, "--slug", help=""""""),
) -> None:
    """Remove assets from a shared link

    [link=https://api.immich.app/endpoints/shared-links/removeSharedLinkAssets]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    if key is not None:
        kwargs["key"] = key
    if slug is not None:
        kwargs["slug"] = slug
    set_nested(json_data, ["asset_ids"], asset_ids)
    asset_ids_dto = AssetIdsDto.model_validate(json_data)
    kwargs["asset_ids_dto"] = asset_ids_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.shared_links, "remove_shared_link_assets", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("update-shared-link", deprecated=False, rich_help_panel="API commands")
def update_shared_link(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    allow_download: Literal["true", "false"] | None = typer.Option(
        None, "--allow-download", help=""""""
    ),
    allow_upload: Literal["true", "false"] | None = typer.Option(
        None, "--allow-upload", help=""""""
    ),
    change_expiry_time: Literal["true", "false"] | None = typer.Option(
        None,
        "--change-expiry-time",
        help="""Few clients cannot send null to set the expiryTime to never.
Setting this flag and not sending expiryAt is considered as null instead.
Clients that can send null values can ignore this.""",
    ),
    description: str | None = typer.Option(None, "--description", help=""""""),
    expires_at: datetime | None = typer.Option(None, "--expires-at", help=""""""),
    password: str | None = typer.Option(None, "--password", help=""""""),
    show_metadata: Literal["true", "false"] | None = typer.Option(
        None, "--show-metadata", help=""""""
    ),
    slug: str | None = typer.Option(None, "--slug", help=""""""),
) -> None:
    """Update a shared link

    [link=https://api.immich.app/endpoints/shared-links/updateSharedLink]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    if allow_download is not None:
        set_nested(json_data, ["allow_download"], allow_download.lower() == "true")
    if allow_upload is not None:
        set_nested(json_data, ["allow_upload"], allow_upload.lower() == "true")
    if change_expiry_time is not None:
        set_nested(
            json_data, ["change_expiry_time"], change_expiry_time.lower() == "true"
        )
    if description is not None:
        set_nested(json_data, ["description"], description)
    if expires_at is not None:
        set_nested(json_data, ["expires_at"], expires_at)
    if password is not None:
        set_nested(json_data, ["password"], password)
    if show_metadata is not None:
        set_nested(json_data, ["show_metadata"], show_metadata.lower() == "true")
    if slug is not None:
        set_nested(json_data, ["slug"], slug)
    shared_link_edit_dto = SharedLinkEditDto.model_validate(json_data)
    kwargs["shared_link_edit_dto"] = shared_link_edit_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.shared_links, "update_shared_link", ctx, **kwargs
    )
    print_response(result, ctx)



from __future__ import annotations

import asyncio
import fnmatch
import hashlib
import json
import logging
import os
import sys
from statx import statx
from datetime import datetime, timezone
from pathlib import Path
from typing import Optional, cast
from uuid import UUID
import uuid

from rich.progress import (
    Progress,
    SpinnerColumn,
    BarColumn,
    TextColumn,
    DownloadColumn,
    TransferSpeedColumn,
    TimeRemainingColumn,
)
from immich.client.consts import DEVICE_ID
from immich.client.types import (
    RejectedEntry,
    FailedEntry,
    UploadedEntry,
    RejectionReason,
)
from immich.client.generated.api.albums_api import AlbumsApi
from immich.client.generated.api.assets_api import AssetsApi
from immich.client.generated.api.server_api import ServerApi
from immich.client.generated.api_response import ApiResponse
from immich.client.generated.models.asset_bulk_upload_check_dto import (
    AssetBulkUploadCheckDto,
)
from immich.client.generated.models.asset_bulk_upload_check_item import (
    AssetBulkUploadCheckItem,
)
from immich.client.generated.models.asset_media_response_dto import (
    AssetMediaResponseDto,
)
from immich.client.generated.models.asset_media_status import AssetMediaStatus
from immich.client.generated.models.bulk_ids_dto import BulkIdsDto
from immich.client.generated.models.create_album_dto import CreateAlbumDto
from immich.client.generated.exceptions import ApiException

logger = logging.getLogger(__name__)

BATCH_SIZE = 5000


def get_device_asset_id(filepath: Path, stats: os.stat_result) -> str:
    """Get the device asset ID for a given file path and stats.

    :param filepath: The path to the file.
    :param stats: The stats of the file.

    :return: The device asset ID.
    """
    return f"{filepath.name}-{stats.st_size}".replace(" ", "")


async def scan_files(
    paths: list[Path],
    server_api: ServerApi,
    ignore_pattern: Optional[str] = None,
    include_hidden: bool = False,
) -> list[Path]:
    """Scan paths for supported media files.

    :param paths: List of file or directory paths to scan.
    :param server_api: Server API instance to query supported media types.
    :param ignore_pattern: Optional glob pattern to ignore matching files.
    :param include_hidden: Whether to include hidden files (starting with .).

    :return: Sorted list of unique file paths that match supported media types.
    """
    media_types = await server_api.get_supported_media_types()
    extensions = set(media_types.image + media_types.video)

    files: list[Path] = []
    for path in paths:
        path = path.resolve()
        if path.is_file():
            if not include_hidden and any(part.startswith(".") for part in path.parts):
                continue
            if path.suffix.lower() in extensions:
                if ignore_pattern and fnmatch.fnmatch(str(path), f"*{ignore_pattern}"):
                    continue
                files.append(path)
        elif path.is_dir():
            for file_path in path.rglob("*"):
                if not file_path.is_file():
                    continue
                if file_path.suffix.lower() not in extensions:
                    continue
                if not include_hidden and any(
                    part.startswith(".") for part in file_path.parts
                ):
                    continue
                if ignore_pattern and fnmatch.fnmatch(
                    str(file_path), f"*{ignore_pattern}"
                ):
                    continue
                files.append(file_path)
    return sorted(set(files))


def compute_sha1_sync(filepath: Path) -> str:  # pragma: no cover
    """Compute SHA1 hash of a file synchronously.

    :param filepath: Path to the file to hash.

    :return: Hexadecimal SHA1 digest string.
    """
    sha1 = hashlib.sha1(usedforsecurity=False)
    with open(filepath, "rb") as f:
        while chunk := f.read(1024 * 1024):
            sha1.update(chunk)
    return sha1.hexdigest()


async def check_duplicates(
    files: list[Path],
    assets_api: AssetsApi,
    skip_duplicates: bool = False,
    show_progress: bool = False,
    dry_run: bool = False,
) -> tuple[list[Path], list[RejectedEntry]]:
    """Check which files are duplicates on the server.

    :param files: List of file paths to check.
    :param assets_api: Assets API instance for duplicate checking.
    :param skip_duplicates: Whether to skip duplicate checking (might still get rejected on the server).
    :param show_progress: Whether to show progress bars.
    :param dry_run: Whether to run in dry run mode (no actual API calls).

    :return: Tuple of (new_files, rejected_entries) where new_files can be uploaded and rejected_entries are duplicates.
    """
    if skip_duplicates or dry_run:
        return files, []

    progress_columns = [
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        BarColumn(),
        TextColumn("[progress.percentage]{task.percentage:>3.0f}%"),
    ]

    with Progress(*progress_columns, disable=not show_progress) as progress:
        hashing_task = progress.add_task("[cyan]Hashing files", total=len(files))
        checksums: list[tuple[Path, str]] = []
        for filepath in files:
            checksum = await asyncio.to_thread(compute_sha1_sync, filepath)
            checksums.append((filepath, checksum))
            progress.update(hashing_task, advance=1)

        new_files: list[Path] = []
        rejected: list[RejectedEntry] = []

        check_task = progress.add_task("[cyan]Checking duplicates", total=len(files))

        for i in range(0, len(checksums), BATCH_SIZE):
            batch = checksums[i : i + BATCH_SIZE]
            items = [
                AssetBulkUploadCheckItem(id=str(filepath), checksum=checksum)
                for filepath, checksum in batch
            ]
            dto = AssetBulkUploadCheckDto(assets=items)
            response = await assets_api.check_bulk_upload(
                asset_bulk_upload_check_dto=dto
            )

            for result in response.results:
                filepath = Path(result.id)
                if result.action == "accept":
                    new_files.append(filepath)
                elif result.action == "reject":
                    rejected.append(
                        RejectedEntry(
                            filepath=filepath,
                            asset_id=result.asset_id,
                            reason=cast(Optional[RejectionReason], result.reason),
                        )
                    )
                else:
                    logger.warning(
                        f"Check upload result returned unexpected action {result.action} for {filepath}"
                    )

            progress.update(check_task, advance=len(batch))

    return new_files, rejected


def find_sidecar(filepath: Path) -> Optional[Path]:
    """Find sidecar file for a given media file path.

    Checks both naming conventions:
    - {filename}.xmp (e.g., photo.xmp for photo.jpg)
    - {filename}.{ext}.xmp (e.g., photo.jpg.xmp for photo.jpg)

    :param filepath: The path to the media file.

    :return: The path to the first sidecar file that exists, or None if neither exists.
    """
    no_ext = filepath.parent / filepath.stem
    for sidecar_path in [
        no_ext.with_suffix(".xmp"),
        filepath.with_suffix(filepath.suffix + ".xmp"),
    ]:
        if sidecar_path.exists():
            return sidecar_path
    return None


def get_file_times(
    path: Path, stats: os.stat_result
) -> tuple[datetime, datetime]:  # pragma: no cover
    """Get file creation and modification times reliably.

    Credits: https://stackoverflow.com/a/39501288

    :param path: The path to the file (used for statx fallback on Linux).
    :param stats: The stat result from os.stat() or Path.stat().

    :return: Tuple of (creation_time, modification_time) as UTC-aware datetimes.
    """
    mtime = stats.st_mtime

    try:
        # not available on all platforms and python versions, thus the AttributeError guard and type ignore
        # ty says to remove the type ignore, however in the ci the lint would raise an error
        ctime = stats.st_birthtime  # type: ignore[unresolved-attribute]
    except AttributeError:
        if sys.platform == "win32":
            ctime = stats.st_ctime
        else:
            ctime = statx(os.fspath(path)).btime

    return datetime.fromtimestamp(
        ctime or mtime, tz=timezone.utc
    ), datetime.fromtimestamp(mtime, tz=timezone.utc)


async def upload_file(
    filepath: Path,
    assets_api: AssetsApi,
    dry_run: bool = False,
) -> ApiResponse[AssetMediaResponseDto]:
    """Upload a single asset file to the server.

    :param filepath: Path to the file to upload.
    :param assets_api: Assets API instance for upload.
    :param dry_run: Return mock response without actual upload.

    :return: API response containing the uploaded asset metadata.
    """
    if dry_run:
        mock_data = AssetMediaResponseDto(
            id=str(uuid.uuid4()), status=AssetMediaStatus.CREATED
        )
        return ApiResponse(
            status_code=201,
            headers=None,
            data=mock_data,
            raw_data=b"",
        )

    stats = filepath.stat()

    sidecar_data: Optional[str] = None
    sidecar_path = find_sidecar(filepath)
    if sidecar_path:
        sidecar_data = str(sidecar_path)

    asset_data = str(filepath)

    file_created_at, file_modified_at = get_file_times(filepath, stats)

    response = await assets_api.upload_asset_with_http_info(
        asset_data=asset_data,
        device_asset_id=get_device_asset_id(filepath, stats),
        device_id=DEVICE_ID,
        file_created_at=file_created_at,
        file_modified_at=file_modified_at,
        sidecar_data=sidecar_data,
    )
    return response


async def upload_files(
    files: list[Path],
    assets_api: AssetsApi,
    concurrency: int = 5,
    show_progress: bool = False,
    dry_run: bool = False,
) -> tuple[list[UploadedEntry], list[RejectedEntry], list[FailedEntry]]:
    """Upload multiple asset files concurrently.

    :param files: List of file paths to upload.
    :param assets_api: Assets API instance for upload.
    :param concurrency: Maximum number of concurrent uploads.
    :param show_progress: Whether to show upload progress bar.
    :param dry_run: Simulate uploads without actual API calls.

    :return: Tuple of (uploaded_entries, rejected_entries, failed_entries).
    """
    if not files:
        return [], [], []

    total_size = sum(f.stat().st_size for f in files)
    progress_columns = [
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        BarColumn(),
        TextColumn("[progress.percentage]{task.percentage:>3.0f}%"),
        DownloadColumn(),
        TransferSpeedColumn(),
        TimeRemainingColumn(),
    ]

    semaphore = asyncio.Semaphore(concurrency)
    uploaded: list[UploadedEntry] = []
    rejected: list[RejectedEntry] = []
    failed: list[FailedEntry] = []

    with Progress(*progress_columns, disable=not show_progress) as progress:
        upload_task = progress.add_task("[green]Uploading assets", total=total_size)

        async def upload_with_semaphore(filepath: Path) -> None:
            async with semaphore:
                try:
                    response = await upload_file(filepath, assets_api, dry_run)
                    if response.status_code == 201:
                        uploaded.append(
                            UploadedEntry(asset=response.data, filepath=filepath)
                        )
                    elif response.status_code == 200:
                        rejected.append(
                            RejectedEntry(
                                filepath=filepath,
                                asset_id=response.data.id,
                                reason="duplicate",
                            )
                        )
                    else:
                        failed.append(
                            FailedEntry(
                                filepath=filepath,
                                error=f"Unexpected status_code={response.status_code}",
                            )
                        )
                    if not dry_run:
                        progress.update(upload_task, advance=filepath.stat().st_size)
                except ApiException as e:
                    msg = str(e)
                    if e.body:
                        try:
                            body = json.loads(cast(str, e.body))
                            msg = str(body.get("message", msg))
                        except Exception:  # nosec: B110
                            pass
                    failed.append(FailedEntry(filepath=filepath, error=msg))
                    logger.exception("Failed to upload %s: %s", filepath, msg)
                except Exception as e:
                    msg = str(e)
                    failed.append(FailedEntry(filepath=filepath, error=msg))
                    logger.exception("Failed to upload %s: %s", filepath, msg)

        await asyncio.gather(*[upload_with_semaphore(f) for f in files])

    return uploaded, rejected, failed


async def update_albums(
    uploaded: list[UploadedEntry],
    album_name: Optional[str],
    albums_api: AlbumsApi,
) -> None:
    """Add uploaded assets to an album, creating the album if needed.

    :param uploaded: List of successfully uploaded entries.
    :param album_name: Name of the album to add assets to (if None, no action).
    :param albums_api: Albums API instance.

    :return: None
    """
    if not album_name or not uploaded:
        return

    all_albums = await albums_api.get_all_albums()
    album_map = {album.album_name: album.id for album in all_albums}

    if album_name not in album_map:
        album = await albums_api.create_album(
            create_album_dto=CreateAlbumDto(albumName=album_name)
        )
        album_map[album_name] = album.id

    album_id = album_map[album_name]
    asset_ids = [UUID(str(entry.asset.id)) for entry in uploaded]

    for i in range(0, len(asset_ids), 1000):
        batch = asset_ids[i : i + 1000]
        await albums_api.add_assets_to_album(
            id=UUID(str(album_id)), bulk_ids_dto=BulkIdsDto(ids=batch)
        )


async def delete_files(
    uploaded: list[UploadedEntry],
    rejected: list[RejectedEntry],
    delete_uploads: bool = False,
    delete_duplicates: bool = False,
    dry_run: bool = False,
) -> None:
    """Delete local files after upload or if they are duplicates.

    :param uploaded: List of successfully uploaded entries.
    :param rejected: List of rejected entries (e.g., duplicates).
    :param delete_uploads: Whether to delete files that were successfully uploaded.
    :param delete_duplicates: Whether to delete files that were rejected as duplicates.
    :param dry_run: Log deletions without actually deleting files.

    :return: None
    """
    to_delete: list[Path] = []
    if delete_uploads:
        for _ in uploaded:
            to_delete.append(_.filepath)

    if delete_duplicates:
        for _ in rejected:
            if _.reason == "duplicate":
                to_delete.append(_.filepath)

    for filepath in to_delete:
        main_deleted = True
        if dry_run:
            logger.info(f"Would have deleted {filepath}")
        else:
            try:
                filepath.unlink()
            except Exception:
                main_deleted = False
                logger.exception(f"Failed to delete {filepath}")

        if main_deleted:
            sidecar_path = find_sidecar(filepath)
            if sidecar_path:
                if dry_run:
                    logger.info(f"Would have deleted {sidecar_path}")
                else:
                    try:
                        sidecar_path.unlink()
                    except Exception:
                        logger.exception(f"Failed to delete sidecar {sidecar_path}")



"""Generated CLI commands for System config tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from typing import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""Endpoints to view, modify, and validate the system configuration settings.\n\n[link=https://api.immich.app/endpoints/system-config]Immich API documentation[/link]"""
)


@app.command("get-config", deprecated=False, rich_help_panel="API commands")
def get_config(
    ctx: typer.Context,
) -> None:
    """Get system configuration

    [link=https://api.immich.app/endpoints/system-config/getConfig]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.system_config, "get_config", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-config-defaults", deprecated=False, rich_help_panel="API commands")
def get_config_defaults(
    ctx: typer.Context,
) -> None:
    """Get system configuration defaults

    [link=https://api.immich.app/endpoints/system-config/getConfigDefaults]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.system_config, "get_config_defaults", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command(
    "get-storage-template-options", deprecated=False, rich_help_panel="API commands"
)
def get_storage_template_options(
    ctx: typer.Context,
) -> None:
    """Get storage template options

    [link=https://api.immich.app/endpoints/system-config/getStorageTemplateOptions]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.system_config, "get_storage_template_options", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("update-config", deprecated=False, rich_help_panel="API commands")
def update_config(
    ctx: typer.Context,
    backup_database_cron_expression: str = typer.Option(
        ..., "--backup-database-cron-expression", help=""""""
    ),
    backup_database_enabled: bool = typer.Option(
        ..., "--backup-database-enabled", help=""""""
    ),
    backup_database_keep_last_amount: float = typer.Option(
        ..., "--backup-database-keep-last-amount", help="""""", min=1
    ),
    ffmpeg_accel: str = typer.Option(..., "--ffmpeg-accel", help=""""""),
    ffmpeg_accel_decode: bool = typer.Option(..., "--ffmpeg-accel-decode", help=""""""),
    ffmpeg_accepted_audio_codecs: list[AudioCodec] = typer.Option(
        ..., "--ffmpeg-accepted-audio-codecs", help=""""""
    ),
    ffmpeg_accepted_containers: list[VideoContainer] = typer.Option(
        ..., "--ffmpeg-accepted-containers", help=""""""
    ),
    ffmpeg_accepted_video_codecs: list[VideoCodec] = typer.Option(
        ..., "--ffmpeg-accepted-video-codecs", help=""""""
    ),
    ffmpeg_bframes: int = typer.Option(
        ..., "--ffmpeg-bframes", help="""""", min=-1, max=16
    ),
    ffmpeg_cq_mode: str = typer.Option(..., "--ffmpeg-cq-mode", help=""""""),
    ffmpeg_crf: int = typer.Option(..., "--ffmpeg-crf", help="""""", min=0, max=51),
    ffmpeg_gop_size: int = typer.Option(..., "--ffmpeg-gop-size", help="""""", min=0),
    ffmpeg_max_bitrate: str = typer.Option(..., "--ffmpeg-max-bitrate", help=""""""),
    ffmpeg_preferred_hw_device: str = typer.Option(
        ..., "--ffmpeg-preferred-hw-device", help=""""""
    ),
    ffmpeg_preset: str = typer.Option(..., "--ffmpeg-preset", help=""""""),
    ffmpeg_refs: int = typer.Option(..., "--ffmpeg-refs", help="""""", min=0, max=6),
    ffmpeg_target_audio_codec: str = typer.Option(
        ..., "--ffmpeg-target-audio-codec", help=""""""
    ),
    ffmpeg_target_resolution: str = typer.Option(
        ..., "--ffmpeg-target-resolution", help=""""""
    ),
    ffmpeg_target_video_codec: str = typer.Option(
        ..., "--ffmpeg-target-video-codec", help=""""""
    ),
    ffmpeg_temporal_aq: bool = typer.Option(..., "--ffmpeg-temporal-aq", help=""""""),
    ffmpeg_threads: int = typer.Option(..., "--ffmpeg-threads", help="""""", min=0),
    ffmpeg_tonemap: str = typer.Option(..., "--ffmpeg-tonemap", help=""""""),
    ffmpeg_transcode: str = typer.Option(..., "--ffmpeg-transcode", help=""""""),
    ffmpeg_two_pass: bool = typer.Option(..., "--ffmpeg-two-pass", help=""""""),
    image_colorspace: str = typer.Option(..., "--image-colorspace", help=""""""),
    image_extract_embedded: bool = typer.Option(
        ..., "--image-extract-embedded", help=""""""
    ),
    image_fullsize_enabled: bool = typer.Option(
        ..., "--image-fullsize-enabled", help=""""""
    ),
    image_fullsize_format: str = typer.Option(
        ..., "--image-fullsize-format", help=""""""
    ),
    image_fullsize_progressive: Literal["true", "false"] | None = typer.Option(
        None, "--image-fullsize-progressive", help=""""""
    ),
    image_fullsize_quality: int = typer.Option(
        ..., "--image-fullsize-quality", help="""""", min=1, max=100
    ),
    image_preview_format: str = typer.Option(
        ..., "--image-preview-format", help=""""""
    ),
    image_preview_progressive: Literal["true", "false"] | None = typer.Option(
        None, "--image-preview-progressive", help=""""""
    ),
    image_preview_quality: int = typer.Option(
        ..., "--image-preview-quality", help="""""", min=1, max=100
    ),
    image_preview_size: int = typer.Option(
        ..., "--image-preview-size", help="""""", min=1
    ),
    image_thumbnail_format: str = typer.Option(
        ..., "--image-thumbnail-format", help=""""""
    ),
    image_thumbnail_progressive: Literal["true", "false"] | None = typer.Option(
        None, "--image-thumbnail-progressive", help=""""""
    ),
    image_thumbnail_quality: int = typer.Option(
        ..., "--image-thumbnail-quality", help="""""", min=1, max=100
    ),
    image_thumbnail_size: int = typer.Option(
        ..., "--image-thumbnail-size", help="""""", min=1
    ),
    job_background_task_concurrency: int = typer.Option(
        ..., "--job-background-task-concurrency", help="""""", min=1
    ),
    job_editor_concurrency: int = typer.Option(
        ..., "--job-editor-concurrency", help="""""", min=1
    ),
    job_face_detection_concurrency: int = typer.Option(
        ..., "--job-face-detection-concurrency", help="""""", min=1
    ),
    job_library_concurrency: int = typer.Option(
        ..., "--job-library-concurrency", help="""""", min=1
    ),
    job_metadata_extraction_concurrency: int = typer.Option(
        ..., "--job-metadata-extraction-concurrency", help="""""", min=1
    ),
    job_migration_concurrency: int = typer.Option(
        ..., "--job-migration-concurrency", help="""""", min=1
    ),
    job_notifications_concurrency: int = typer.Option(
        ..., "--job-notifications-concurrency", help="""""", min=1
    ),
    job_ocr_concurrency: int = typer.Option(
        ..., "--job-ocr-concurrency", help="""""", min=1
    ),
    job_search_concurrency: int = typer.Option(
        ..., "--job-search-concurrency", help="""""", min=1
    ),
    job_sidecar_concurrency: int = typer.Option(
        ..., "--job-sidecar-concurrency", help="""""", min=1
    ),
    job_smart_search_concurrency: int = typer.Option(
        ..., "--job-smart-search-concurrency", help="""""", min=1
    ),
    job_thumbnail_generation_concurrency: int = typer.Option(
        ..., "--job-thumbnail-generation-concurrency", help="""""", min=1
    ),
    job_video_conversion_concurrency: int = typer.Option(
        ..., "--job-video-conversion-concurrency", help="""""", min=1
    ),
    job_workflow_concurrency: int = typer.Option(
        ..., "--job-workflow-concurrency", help="""""", min=1
    ),
    library_scan_cron_expression: str = typer.Option(
        ..., "--library-scan-cron-expression", help=""""""
    ),
    library_scan_enabled: bool = typer.Option(
        ..., "--library-scan-enabled", help=""""""
    ),
    library_watch_enabled: bool = typer.Option(
        ..., "--library-watch-enabled", help=""""""
    ),
    logging_enabled: bool = typer.Option(..., "--logging-enabled", help=""""""),
    logging_level: str = typer.Option(..., "--logging-level", help=""""""),
    machine_learning_availability_checks_enabled: bool = typer.Option(
        ..., "--machine-learning-availability-checks-enabled", help=""""""
    ),
    machine_learning_availability_checks_interval: float = typer.Option(
        ..., "--machine-learning-availability-checks-interval", help=""""""
    ),
    machine_learning_availability_checks_timeout: float = typer.Option(
        ..., "--machine-learning-availability-checks-timeout", help=""""""
    ),
    machine_learning_clip_enabled: bool = typer.Option(
        ..., "--machine-learning-clip-enabled", help=""""""
    ),
    machine_learning_clip_model_name: str = typer.Option(
        ..., "--machine-learning-clip-model-name", help=""""""
    ),
    machine_learning_duplicate_detection_enabled: bool = typer.Option(
        ..., "--machine-learning-duplicate-detection-enabled", help=""""""
    ),
    machine_learning_duplicate_detection_max_distance: float = typer.Option(
        ...,
        "--machine-learning-duplicate-detection-max-distance",
        help="""""",
        min=0.001,
        max=0.1,
    ),
    machine_learning_enabled: bool = typer.Option(
        ..., "--machine-learning-enabled", help=""""""
    ),
    machine_learning_facial_recognition_enabled: bool = typer.Option(
        ..., "--machine-learning-facial-recognition-enabled", help=""""""
    ),
    machine_learning_facial_recognition_max_distance: float = typer.Option(
        ...,
        "--machine-learning-facial-recognition-max-distance",
        help="""""",
        min=0.1,
        max=2,
    ),
    machine_learning_facial_recognition_min_faces: int = typer.Option(
        ..., "--machine-learning-facial-recognition-min-faces", help="""""", min=1
    ),
    machine_learning_facial_recognition_min_score: float = typer.Option(
        ...,
        "--machine-learning-facial-recognition-min-score",
        help="""""",
        min=0.1,
        max=1,
    ),
    machine_learning_facial_recognition_model_name: str = typer.Option(
        ..., "--machine-learning-facial-recognition-model-name", help=""""""
    ),
    machine_learning_ocr_enabled: bool = typer.Option(
        ..., "--machine-learning-ocr-enabled", help=""""""
    ),
    machine_learning_ocr_max_resolution: int = typer.Option(
        ..., "--machine-learning-ocr-max-resolution", help="""""", min=1
    ),
    machine_learning_ocr_min_detection_score: float = typer.Option(
        ..., "--machine-learning-ocr-min-detection-score", help="""""", min=0.1, max=1
    ),
    machine_learning_ocr_min_recognition_score: float = typer.Option(
        ..., "--machine-learning-ocr-min-recognition-score", help="""""", min=0.1, max=1
    ),
    machine_learning_ocr_model_name: str = typer.Option(
        ..., "--machine-learning-ocr-model-name", help=""""""
    ),
    machine_learning_urls: list[str] = typer.Option(
        ..., "--machine-learning-urls", help=""""""
    ),
    map_dark_style: str = typer.Option(..., "--map-dark-style", help=""""""),
    map_enabled: bool = typer.Option(..., "--map-enabled", help=""""""),
    map_light_style: str = typer.Option(..., "--map-light-style", help=""""""),
    metadata_faces_import_: bool = typer.Option(
        ..., "--metadata-faces-import", help=""""""
    ),
    new_version_check_enabled: bool = typer.Option(
        ..., "--new-version-check-enabled", help=""""""
    ),
    nightly_tasks_cluster_new_faces: bool = typer.Option(
        ..., "--nightly-tasks-cluster-new-faces", help=""""""
    ),
    nightly_tasks_database_cleanup: bool = typer.Option(
        ..., "--nightly-tasks-database-cleanup", help=""""""
    ),
    nightly_tasks_generate_memories: bool = typer.Option(
        ..., "--nightly-tasks-generate-memories", help=""""""
    ),
    nightly_tasks_missing_thumbnails: bool = typer.Option(
        ..., "--nightly-tasks-missing-thumbnails", help=""""""
    ),
    nightly_tasks_start_time: str = typer.Option(
        ..., "--nightly-tasks-start-time", help=""""""
    ),
    nightly_tasks_sync_quota_usage: bool = typer.Option(
        ..., "--nightly-tasks-sync-quota-usage", help=""""""
    ),
    notifications_smtp_enabled: bool = typer.Option(
        ..., "--notifications-smtp-enabled", help=""""""
    ),
    notifications_smtp_from_: str = typer.Option(
        ..., "--notifications-smtp-from", help=""""""
    ),
    notifications_smtp_reply_to: str = typer.Option(
        ..., "--notifications-smtp-reply-to", help=""""""
    ),
    notifications_smtp_transport_host: str = typer.Option(
        ..., "--notifications-smtp-transport-host", help=""""""
    ),
    notifications_smtp_transport_ignore_cert: bool = typer.Option(
        ..., "--notifications-smtp-transport-ignore-cert", help=""""""
    ),
    notifications_smtp_transport_password: str = typer.Option(
        ..., "--notifications-smtp-transport-password", help=""""""
    ),
    notifications_smtp_transport_port: float = typer.Option(
        ..., "--notifications-smtp-transport-port", help="""""", min=0, max=65535
    ),
    notifications_smtp_transport_secure: bool = typer.Option(
        ..., "--notifications-smtp-transport-secure", help=""""""
    ),
    notifications_smtp_transport_username: str = typer.Option(
        ..., "--notifications-smtp-transport-username", help=""""""
    ),
    oauth_auto_launch: bool = typer.Option(..., "--oauth-auto-launch", help=""""""),
    oauth_auto_register: bool = typer.Option(..., "--oauth-auto-register", help=""""""),
    oauth_button_text: str = typer.Option(..., "--oauth-button-text", help=""""""),
    oauth_client_id: str = typer.Option(..., "--oauth-client-id", help=""""""),
    oauth_client_secret: str = typer.Option(..., "--oauth-client-secret", help=""""""),
    oauth_default_storage_quota: int = typer.Option(
        ..., "--oauth-default-storage-quota", help="""""", min=0
    ),
    oauth_enabled: bool = typer.Option(..., "--oauth-enabled", help=""""""),
    oauth_issuer_url: str = typer.Option(..., "--oauth-issuer-url", help=""""""),
    oauth_mobile_override_enabled: bool = typer.Option(
        ..., "--oauth-mobile-override-enabled", help=""""""
    ),
    oauth_mobile_redirect_uri: str = typer.Option(
        ..., "--oauth-mobile-redirect-uri", help=""""""
    ),
    oauth_profile_signing_algorithm: str = typer.Option(
        ..., "--oauth-profile-signing-algorithm", help=""""""
    ),
    oauth_role_claim: str = typer.Option(..., "--oauth-role-claim", help=""""""),
    oauth_scope: str = typer.Option(..., "--oauth-scope", help=""""""),
    oauth_signing_algorithm: str = typer.Option(
        ..., "--oauth-signing-algorithm", help=""""""
    ),
    oauth_storage_label_claim: str = typer.Option(
        ..., "--oauth-storage-label-claim", help=""""""
    ),
    oauth_storage_quota_claim: str = typer.Option(
        ..., "--oauth-storage-quota-claim", help=""""""
    ),
    oauth_timeout: int = typer.Option(..., "--oauth-timeout", help="""""", min=1),
    oauth_token_endpoint_auth_method: str = typer.Option(
        ..., "--oauth-token-endpoint-auth-method", help=""""""
    ),
    password_login_enabled: bool = typer.Option(
        ..., "--password-login-enabled", help=""""""
    ),
    reverse_geocoding_enabled: bool = typer.Option(
        ..., "--reverse-geocoding-enabled", help=""""""
    ),
    server_external_domain: str = typer.Option(
        ..., "--server-external-domain", help=""""""
    ),
    server_login_page_message: str = typer.Option(
        ..., "--server-login-page-message", help=""""""
    ),
    server_public_users: bool = typer.Option(..., "--server-public-users", help=""""""),
    storage_template_enabled: bool = typer.Option(
        ..., "--storage-template-enabled", help=""""""
    ),
    storage_template_hash_verification_enabled: bool = typer.Option(
        ..., "--storage-template-hash-verification-enabled", help=""""""
    ),
    storage_template_template: str = typer.Option(
        ..., "--storage-template-template", help=""""""
    ),
    templates_email_album_invite_template: str = typer.Option(
        ..., "--templates-email-album-invite-template", help=""""""
    ),
    templates_email_album_update_template: str = typer.Option(
        ..., "--templates-email-album-update-template", help=""""""
    ),
    templates_email_welcome_template: str = typer.Option(
        ..., "--templates-email-welcome-template", help=""""""
    ),
    theme_custom_css: str = typer.Option(..., "--theme-custom-css", help=""""""),
    trash_days: int = typer.Option(..., "--trash-days", help="""""", min=0),
    trash_enabled: bool = typer.Option(..., "--trash-enabled", help=""""""),
    user_delete_delay: int = typer.Option(
        ..., "--user-delete-delay", help="""""", min=1
    ),
) -> None:
    """Update system configuration

    [link=https://api.immich.app/endpoints/system-config/updateConfig]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(
        json_data, ["backup_database_cron_expression"], backup_database_cron_expression
    )
    set_nested(json_data, ["backup_database_enabled"], backup_database_enabled)
    set_nested(
        json_data,
        ["backup_database_keep_last_amount"],
        backup_database_keep_last_amount,
    )
    set_nested(json_data, ["ffmpeg_accel"], ffmpeg_accel)
    set_nested(json_data, ["ffmpeg_accel_decode"], ffmpeg_accel_decode)
    set_nested(
        json_data, ["ffmpeg_accepted_audio_codecs"], ffmpeg_accepted_audio_codecs
    )
    set_nested(json_data, ["ffmpeg_accepted_containers"], ffmpeg_accepted_containers)
    set_nested(
        json_data, ["ffmpeg_accepted_video_codecs"], ffmpeg_accepted_video_codecs
    )
    set_nested(json_data, ["ffmpeg_bframes"], ffmpeg_bframes)
    set_nested(json_data, ["ffmpeg_cq_mode"], ffmpeg_cq_mode)
    set_nested(json_data, ["ffmpeg_crf"], ffmpeg_crf)
    set_nested(json_data, ["ffmpeg_gop_size"], ffmpeg_gop_size)
    set_nested(json_data, ["ffmpeg_max_bitrate"], ffmpeg_max_bitrate)
    set_nested(json_data, ["ffmpeg_preferred_hw_device"], ffmpeg_preferred_hw_device)
    set_nested(json_data, ["ffmpeg_preset"], ffmpeg_preset)
    set_nested(json_data, ["ffmpeg_refs"], ffmpeg_refs)
    set_nested(json_data, ["ffmpeg_target_audio_codec"], ffmpeg_target_audio_codec)
    set_nested(json_data, ["ffmpeg_target_resolution"], ffmpeg_target_resolution)
    set_nested(json_data, ["ffmpeg_target_video_codec"], ffmpeg_target_video_codec)
    set_nested(json_data, ["ffmpeg_temporal_aq"], ffmpeg_temporal_aq)
    set_nested(json_data, ["ffmpeg_threads"], ffmpeg_threads)
    set_nested(json_data, ["ffmpeg_tonemap"], ffmpeg_tonemap)
    set_nested(json_data, ["ffmpeg_transcode"], ffmpeg_transcode)
    set_nested(json_data, ["ffmpeg_two_pass"], ffmpeg_two_pass)
    set_nested(json_data, ["image_colorspace"], image_colorspace)
    set_nested(json_data, ["image_extract_embedded"], image_extract_embedded)
    set_nested(json_data, ["image_fullsize_enabled"], image_fullsize_enabled)
    set_nested(json_data, ["image_fullsize_format"], image_fullsize_format)
    if image_fullsize_progressive is not None:
        set_nested(
            json_data,
            ["image_fullsize_progressive"],
            image_fullsize_progressive.lower() == "true",
        )
    set_nested(json_data, ["image_fullsize_quality"], image_fullsize_quality)
    set_nested(json_data, ["image_preview_format"], image_preview_format)
    if image_preview_progressive is not None:
        set_nested(
            json_data,
            ["image_preview_progressive"],
            image_preview_progressive.lower() == "true",
        )
    set_nested(json_data, ["image_preview_quality"], image_preview_quality)
    set_nested(json_data, ["image_preview_size"], image_preview_size)
    set_nested(json_data, ["image_thumbnail_format"], image_thumbnail_format)
    if image_thumbnail_progressive is not None:
        set_nested(
            json_data,
            ["image_thumbnail_progressive"],
            image_thumbnail_progressive.lower() == "true",
        )
    set_nested(json_data, ["image_thumbnail_quality"], image_thumbnail_quality)
    set_nested(json_data, ["image_thumbnail_size"], image_thumbnail_size)
    set_nested(
        json_data, ["job_background_task_concurrency"], job_background_task_concurrency
    )
    set_nested(json_data, ["job_editor_concurrency"], job_editor_concurrency)
    set_nested(
        json_data, ["job_face_detection_concurrency"], job_face_detection_concurrency
    )
    set_nested(json_data, ["job_library_concurrency"], job_library_concurrency)
    set_nested(
        json_data,
        ["job_metadata_extraction_concurrency"],
        job_metadata_extraction_concurrency,
    )
    set_nested(json_data, ["job_migration_concurrency"], job_migration_concurrency)
    set_nested(
        json_data, ["job_notifications_concurrency"], job_notifications_concurrency
    )
    set_nested(json_data, ["job_ocr_concurrency"], job_ocr_concurrency)
    set_nested(json_data, ["job_search_concurrency"], job_search_concurrency)
    set_nested(json_data, ["job_sidecar_concurrency"], job_sidecar_concurrency)
    set_nested(
        json_data, ["job_smart_search_concurrency"], job_smart_search_concurrency
    )
    set_nested(
        json_data,
        ["job_thumbnail_generation_concurrency"],
        job_thumbnail_generation_concurrency,
    )
    set_nested(
        json_data,
        ["job_video_conversion_concurrency"],
        job_video_conversion_concurrency,
    )
    set_nested(json_data, ["job_workflow_concurrency"], job_workflow_concurrency)
    set_nested(
        json_data, ["library_scan_cron_expression"], library_scan_cron_expression
    )
    set_nested(json_data, ["library_scan_enabled"], library_scan_enabled)
    set_nested(json_data, ["library_watch_enabled"], library_watch_enabled)
    set_nested(json_data, ["logging_enabled"], logging_enabled)
    set_nested(json_data, ["logging_level"], logging_level)
    set_nested(
        json_data,
        ["machine_learning_availability_checks_enabled"],
        machine_learning_availability_checks_enabled,
    )
    set_nested(
        json_data,
        ["machine_learning_availability_checks_interval"],
        machine_learning_availability_checks_interval,
    )
    set_nested(
        json_data,
        ["machine_learning_availability_checks_timeout"],
        machine_learning_availability_checks_timeout,
    )
    set_nested(
        json_data, ["machine_learning_clip_enabled"], machine_learning_clip_enabled
    )
    set_nested(
        json_data,
        ["machine_learning_clip_model_name"],
        machine_learning_clip_model_name,
    )
    set_nested(
        json_data,
        ["machine_learning_duplicate_detection_enabled"],
        machine_learning_duplicate_detection_enabled,
    )
    set_nested(
        json_data,
        ["machine_learning_duplicate_detection_max_distance"],
        machine_learning_duplicate_detection_max_distance,
    )
    set_nested(json_data, ["machine_learning_enabled"], machine_learning_enabled)
    set_nested(
        json_data,
        ["machine_learning_facial_recognition_enabled"],
        machine_learning_facial_recognition_enabled,
    )
    set_nested(
        json_data,
        ["machine_learning_facial_recognition_max_distance"],
        machine_learning_facial_recognition_max_distance,
    )
    set_nested(
        json_data,
        ["machine_learning_facial_recognition_min_faces"],
        machine_learning_facial_recognition_min_faces,
    )
    set_nested(
        json_data,
        ["machine_learning_facial_recognition_min_score"],
        machine_learning_facial_recognition_min_score,
    )
    set_nested(
        json_data,
        ["machine_learning_facial_recognition_model_name"],
        machine_learning_facial_recognition_model_name,
    )
    set_nested(
        json_data, ["machine_learning_ocr_enabled"], machine_learning_ocr_enabled
    )
    set_nested(
        json_data,
        ["machine_learning_ocr_max_resolution"],
        machine_learning_ocr_max_resolution,
    )
    set_nested(
        json_data,
        ["machine_learning_ocr_min_detection_score"],
        machine_learning_ocr_min_detection_score,
    )
    set_nested(
        json_data,
        ["machine_learning_ocr_min_recognition_score"],
        machine_learning_ocr_min_recognition_score,
    )
    set_nested(
        json_data, ["machine_learning_ocr_model_name"], machine_learning_ocr_model_name
    )
    set_nested(json_data, ["machine_learning_urls"], machine_learning_urls)
    set_nested(json_data, ["map_dark_style"], map_dark_style)
    set_nested(json_data, ["map_enabled"], map_enabled)
    set_nested(json_data, ["map_light_style"], map_light_style)
    set_nested(json_data, ["metadata_faces_import_"], metadata_faces_import_)
    set_nested(json_data, ["new_version_check_enabled"], new_version_check_enabled)
    set_nested(
        json_data, ["nightly_tasks_cluster_new_faces"], nightly_tasks_cluster_new_faces
    )
    set_nested(
        json_data, ["nightly_tasks_database_cleanup"], nightly_tasks_database_cleanup
    )
    set_nested(
        json_data, ["nightly_tasks_generate_memories"], nightly_tasks_generate_memories
    )
    set_nested(
        json_data,
        ["nightly_tasks_missing_thumbnails"],
        nightly_tasks_missing_thumbnails,
    )
    set_nested(json_data, ["nightly_tasks_start_time"], nightly_tasks_start_time)
    set_nested(
        json_data, ["nightly_tasks_sync_quota_usage"], nightly_tasks_sync_quota_usage
    )
    set_nested(json_data, ["notifications_smtp_enabled"], notifications_smtp_enabled)
    set_nested(json_data, ["notifications_smtp_from_"], notifications_smtp_from_)
    set_nested(json_data, ["notifications_smtp_reply_to"], notifications_smtp_reply_to)
    set_nested(
        json_data,
        ["notifications_smtp_transport_host"],
        notifications_smtp_transport_host,
    )
    set_nested(
        json_data,
        ["notifications_smtp_transport_ignore_cert"],
        notifications_smtp_transport_ignore_cert,
    )
    set_nested(
        json_data,
        ["notifications_smtp_transport_password"],
        notifications_smtp_transport_password,
    )
    set_nested(
        json_data,
        ["notifications_smtp_transport_port"],
        notifications_smtp_transport_port,
    )
    set_nested(
        json_data,
        ["notifications_smtp_transport_secure"],
        notifications_smtp_transport_secure,
    )
    set_nested(
        json_data,
        ["notifications_smtp_transport_username"],
        notifications_smtp_transport_username,
    )
    set_nested(json_data, ["oauth_auto_launch"], oauth_auto_launch)
    set_nested(json_data, ["oauth_auto_register"], oauth_auto_register)
    set_nested(json_data, ["oauth_button_text"], oauth_button_text)
    set_nested(json_data, ["oauth_client_id"], oauth_client_id)
    set_nested(json_data, ["oauth_client_secret"], oauth_client_secret)
    set_nested(json_data, ["oauth_default_storage_quota"], oauth_default_storage_quota)
    set_nested(json_data, ["oauth_enabled"], oauth_enabled)
    set_nested(json_data, ["oauth_issuer_url"], oauth_issuer_url)
    set_nested(
        json_data, ["oauth_mobile_override_enabled"], oauth_mobile_override_enabled
    )
    set_nested(json_data, ["oauth_mobile_redirect_uri"], oauth_mobile_redirect_uri)
    set_nested(
        json_data, ["oauth_profile_signing_algorithm"], oauth_profile_signing_algorithm
    )
    set_nested(json_data, ["oauth_role_claim"], oauth_role_claim)
    set_nested(json_data, ["oauth_scope"], oauth_scope)
    set_nested(json_data, ["oauth_signing_algorithm"], oauth_signing_algorithm)
    set_nested(json_data, ["oauth_storage_label_claim"], oauth_storage_label_claim)
    set_nested(json_data, ["oauth_storage_quota_claim"], oauth_storage_quota_claim)
    set_nested(json_data, ["oauth_timeout"], oauth_timeout)
    set_nested(
        json_data,
        ["oauth_token_endpoint_auth_method"],
        oauth_token_endpoint_auth_method,
    )
    set_nested(json_data, ["password_login_enabled"], password_login_enabled)
    set_nested(json_data, ["reverse_geocoding_enabled"], reverse_geocoding_enabled)
    set_nested(json_data, ["server_external_domain"], server_external_domain)
    set_nested(json_data, ["server_login_page_message"], server_login_page_message)
    set_nested(json_data, ["server_public_users"], server_public_users)
    set_nested(json_data, ["storage_template_enabled"], storage_template_enabled)
    set_nested(
        json_data,
        ["storage_template_hash_verification_enabled"],
        storage_template_hash_verification_enabled,
    )
    set_nested(json_data, ["storage_template_template"], storage_template_template)
    set_nested(
        json_data,
        ["templates_email_album_invite_template"],
        templates_email_album_invite_template,
    )
    set_nested(
        json_data,
        ["templates_email_album_update_template"],
        templates_email_album_update_template,
    )
    set_nested(
        json_data,
        ["templates_email_welcome_template"],
        templates_email_welcome_template,
    )
    set_nested(json_data, ["theme_custom_css"], theme_custom_css)
    set_nested(json_data, ["trash_days"], trash_days)
    set_nested(json_data, ["trash_enabled"], trash_enabled)
    set_nested(json_data, ["user_delete_delay"], user_delete_delay)
    system_config_dto = SystemConfigDto.model_validate(json_data)
    kwargs["system_config_dto"] = system_config_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.system_config, "update_config", ctx, **kwargs)
    print_response(result, ctx)



"""Main CLI entrypoint."""

from __future__ import annotations

import sys
import typer
import click
from typing import Optional
from importlib.metadata import version

from rich.console import Console

from immich.cli.consts import (
    API_KEY_URL,
    DEFAULT_FORMAT,
    DEFAULT_PROFILE,
    IMMICH_API_KEY,
    IMMICH_ACCESS_TOKEN,
    IMMICH_API_URL,
    IMMICH_FORMAT,
    IMMICH_PROFILE,
)
from immich.cli.utils import resolve_client_config, mask, print_

from immich import AsyncClient
from immich.cli.types import FormatMode, ClientConfig

# Import command modules
from immich.cli.commands import api_keys as api_keys_commands
from immich.cli.commands import activities as activities_commands
from immich.cli.commands import albums as albums_commands
from immich.cli.wrapper import assets as assets_wrapper
from immich.cli.commands import authentication as authentication_commands
from immich.cli.commands import authentication_admin as authentication_admin_commands
from immich.cli.wrapper import download as download_wrapper
from immich.cli.wrapper import config as config_commands
from immich.cli.wrapper import setup as setup_commands
from immich.cli.commands import duplicates as duplicates_commands
from immich.cli.commands import faces as faces_commands
from immich.cli.commands import jobs as jobs_commands
from immich.cli.commands import libraries as libraries_commands
from immich.cli.commands import maintenance_admin as maintenance_admin_commands
from immich.cli.commands import map as map_commands
from immich.cli.commands import memories as memories_commands
from immich.cli.commands import notifications as notifications_commands
from immich.cli.commands import notifications_admin as notifications_admin_commands
from immich.cli.commands import partners as partners_commands
from immich.cli.commands import people as people_commands
from immich.cli.commands import plugins as plugins_commands
from immich.cli.commands import queues as queues_commands
from immich.cli.commands import search as search_commands
from immich.cli.commands import server as server_commands
from immich.cli.commands import sessions as sessions_commands
from immich.cli.commands import shared_links as shared_links_commands
from immich.cli.commands import stacks as stacks_commands
from immich.cli.commands import sync as sync_commands
from immich.cli.commands import system_config as system_config_commands
from immich.cli.commands import system_metadata as system_metadata_commands
from immich.cli.commands import tags as tags_commands
from immich.cli.commands import timeline as timeline_commands
from immich.cli.commands import trash as trash_commands
from immich.cli.wrapper import users as users_wrapper
from immich.cli.commands import users_admin as users_admin_commands
from immich.cli.commands import views as views_commands
from immich.cli.commands import workflows as workflows_commands
from immich.cli.commands import (
    database_backups_admin as database_backups_admin_commands,
)

# Global state
app = typer.Typer(
    context_settings={"help_option_names": ["-h", "--help"]}, no_args_is_help=True
)
console = Console()
stderr_console = Console(file=sys.stderr)

# Add command modules to the main app
app.add_typer(api_keys_commands.app, name="api-keys", rich_help_panel="API commands")
app.add_typer(
    activities_commands.app, name="activities", rich_help_panel="API commands"
)
app.add_typer(albums_commands.app, name="albums", rich_help_panel="API commands")
app.add_typer(assets_wrapper.app, name="assets", rich_help_panel="API commands")
app.add_typer(authentication_commands.app, name="auth", rich_help_panel="API commands")
app.add_typer(
    authentication_admin_commands.app,
    name="auth-admin",
    rich_help_panel="API commands",
)
app.add_typer(download_wrapper.app, name="download", rich_help_panel="API commands")
app.add_typer(config_commands.app, name="config", rich_help_panel="Custom commands")
app.command(rich_help_panel="Custom commands")(setup_commands.setup)
app.add_typer(
    duplicates_commands.app, name="duplicates", rich_help_panel="API commands"
)
app.add_typer(faces_commands.app, name="faces", rich_help_panel="API commands")
app.add_typer(jobs_commands.app, name="jobs", rich_help_panel="API commands")
app.add_typer(libraries_commands.app, name="libraries", rich_help_panel="API commands")
app.add_typer(
    maintenance_admin_commands.app,
    name="maintenance-admin",
    rich_help_panel="API commands",
)
app.add_typer(map_commands.app, name="map", rich_help_panel="API commands")
app.add_typer(memories_commands.app, name="memories", rich_help_panel="API commands")
app.add_typer(
    notifications_commands.app, name="notifications", rich_help_panel="API commands"
)
app.add_typer(
    notifications_admin_commands.app,
    name="notifications-admin",
    rich_help_panel="API commands",
)
app.add_typer(partners_commands.app, name="partners", rich_help_panel="API commands")
app.add_typer(people_commands.app, name="people", rich_help_panel="API commands")
app.add_typer(plugins_commands.app, name="plugins", rich_help_panel="API commands")
app.add_typer(queues_commands.app, name="queues", rich_help_panel="API commands")
app.add_typer(search_commands.app, name="search", rich_help_panel="API commands")
app.add_typer(server_commands.app, name="server", rich_help_panel="API commands")
app.add_typer(sessions_commands.app, name="sessions", rich_help_panel="API commands")
app.add_typer(
    shared_links_commands.app, name="shared-links", rich_help_panel="API commands"
)
app.add_typer(stacks_commands.app, name="stacks", rich_help_panel="API commands")
app.add_typer(sync_commands.app, name="sync", rich_help_panel="API commands")
app.add_typer(
    system_config_commands.app, name="system-config", rich_help_panel="API commands"
)
app.add_typer(
    system_metadata_commands.app, name="system-metadata", rich_help_panel="API commands"
)
app.add_typer(tags_commands.app, name="tags", rich_help_panel="API commands")
app.add_typer(timeline_commands.app, name="timeline", rich_help_panel="API commands")
app.add_typer(trash_commands.app, name="trash", rich_help_panel="API commands")
app.add_typer(users_wrapper.app, name="users", rich_help_panel="API commands")
app.add_typer(
    users_admin_commands.app, name="users-admin", rich_help_panel="API commands"
)
app.add_typer(views_commands.app, name="views", rich_help_panel="API commands")
app.add_typer(workflows_commands.app, name="workflows", rich_help_panel="API commands")
app.add_typer(
    database_backups_admin_commands.app,
    name="backups",
    rich_help_panel="API commands",
)


def version_callback(value: bool) -> None:  # pragma: no cover
    if value:
        print_(f"immich CLI (unofficial) {version('immich')}", type="text")
        raise typer.Exit(0)


@app.callback(invoke_without_command=False)
def callback(
    ctx: typer.Context,
    verbose: bool = typer.Option(
        False,
        "--verbose",
        "-v",
        help="Show verbose output.",
    ),
    format_mode: FormatMode = typer.Option(
        DEFAULT_FORMAT,
        "--format",
        help="Output format of the CLI.",
        envvar=IMMICH_FORMAT,
    ),
    api_key: Optional[str] = typer.Option(
        None,
        "--api-key",
        help=f"Authorize via API key (get one [link={API_KEY_URL}]here[/link]).",
        envvar=IMMICH_API_KEY,
    ),
    access_token: Optional[str] = typer.Option(
        None,
        "--access-token",
        help="Authorize via access token.",
        envvar=IMMICH_ACCESS_TOKEN,
    ),
    base_url: Optional[str] = typer.Option(
        None,
        "--base-url",
        help="The server to connect to.",
        envvar=IMMICH_API_URL,
    ),
    profile: str = typer.Option(
        DEFAULT_PROFILE,
        "--profile",
        "-p",
        envvar=IMMICH_PROFILE,
        help="The profile to use.",
    ),
    _version: bool = typer.Option(
        False,
        "--version",
        callback=version_callback,
        is_eager=True,
        help="Show version and exit.",
    ),
) -> None:  # pragma: no cover
    ctx.ensure_object(dict)
    ctx.obj["format"] = format_mode
    ctx.obj["verbose"] = verbose
    if ctx.invoked_subcommand is not None and ctx.invoked_subcommand not in [
        "setup",
        "config",
    ]:
        config = resolve_client_config(
            ClientConfig(
                api_key=api_key,
                access_token=access_token,
                base_url=base_url,
            ),
            profile=profile,
            # we only consider the profile explicit if it was set via the command line
            # environment variables are not considered explicit
            profile_explicit=ctx.get_parameter_source("profile")
            == click.core.ParameterSource.COMMANDLINE,
        )
        if not config.base_url:
            print_(
                "No base URL provided. Run 'immich setup' to set up a profile or use '--base-url' to specify a base URL.",
                type="error",
            )
            raise typer.Exit(code=1)
        if ctx.obj["verbose"]:
            cli_vars = {
                k: v
                for k, v in ctx.params.items()
                if k in ClientConfig.model_fields.keys() and v is not None
            }
            print_("Configuration used:", type="debug", ctx=ctx)
            for field in ClientConfig.model_fields.keys():
                value = getattr(config, field)
                if field in ("api_key", "access_token") and value:
                    value = mask(value)
                elif value is None:
                    value = "None"
                source = "cli/env" if field in cli_vars else f"profile '{profile}'"
                print_(f"- {field}: {value} (from {source})", type="debug", ctx=ctx)
        if omit_access_token := (config.api_key is not None):
            print_(
                "Omitting access token because API key is provided.",
                type="debug",
                ctx=ctx,
            )
        ctx.obj["client"] = AsyncClient(
            api_key=config.api_key,
            access_token=None if omit_access_token else config.access_token,
            base_url=config.base_url,
        )


if __name__ == "__main__":  # pragma: no cover
    app()



"""Generated CLI commands for Map tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from datetime import datetime
from typing import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command
from immich.client.generated.models import *

app = typer.Typer(
    help="""Map endpoints include supplemental functionality related to geolocation, such as reverse geocoding and retrieving map markers for assets with geolocation data.\n\n[link=https://api.immich.app/endpoints/map]Immich API documentation[/link]"""
)


@app.command("get-map-markers", deprecated=False, rich_help_panel="API commands")
def get_map_markers(
    ctx: typer.Context,
    file_created_after: datetime | None = typer.Option(
        None, "--file-created-after", help=""""""
    ),
    file_created_before: datetime | None = typer.Option(
        None, "--file-created-before", help=""""""
    ),
    is_archived: Literal["true", "false"] | None = typer.Option(
        None, "--is-archived", help=""""""
    ),
    is_favorite: Literal["true", "false"] | None = typer.Option(
        None, "--is-favorite", help=""""""
    ),
    with_partners: Literal["true", "false"] | None = typer.Option(
        None, "--with-partners", help=""""""
    ),
    with_shared_albums: Literal["true", "false"] | None = typer.Option(
        None, "--with-shared-albums", help=""""""
    ),
) -> None:
    """Retrieve map markers

    [link=https://api.immich.app/endpoints/map/getMapMarkers]Immich API documentation[/link]
    """
    kwargs = {}
    if file_created_after is not None:
        kwargs["file_created_after"] = file_created_after
    if file_created_before is not None:
        kwargs["file_created_before"] = file_created_before
    if is_archived is not None:
        kwargs["is_archived"] = is_archived.lower() == "true"
    if is_favorite is not None:
        kwargs["is_favorite"] = is_favorite.lower() == "true"
    if with_partners is not None:
        kwargs["with_partners"] = with_partners.lower() == "true"
    if with_shared_albums is not None:
        kwargs["with_shared_albums"] = with_shared_albums.lower() == "true"
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.map, "get_map_markers", ctx, **kwargs)
    print_response(result, ctx)


@app.command("reverse-geocode", deprecated=False, rich_help_panel="API commands")
def reverse_geocode(
    ctx: typer.Context,
    lat: float = typer.Option(..., "--lat", help=""""""),
    lon: float = typer.Option(..., "--lon", help=""""""),
) -> None:
    """Reverse geocode coordinates

    [link=https://api.immich.app/endpoints/map/reverseGeocode]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["lat"] = lat
    kwargs["lon"] = lon
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.map, "reverse_geocode", ctx, **kwargs)
    print_response(result, ctx)



"""Generated CLI commands for Duplicates tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""Endpoints for managing and identifying duplicate assets.\n\n[link=https://api.immich.app/endpoints/duplicates]Immich API documentation[/link]"""
)


@app.command("delete-duplicate", deprecated=False, rich_help_panel="API commands")
def delete_duplicate(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Delete a duplicate

    [link=https://api.immich.app/endpoints/duplicates/deleteDuplicate]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.duplicates, "delete_duplicate", ctx, **kwargs)
    print_response(result, ctx)


@app.command("delete-duplicates", deprecated=False, rich_help_panel="API commands")
def delete_duplicates(
    ctx: typer.Context,
    ids: list[str] = typer.Option(..., "--ids", help=""""""),
) -> None:
    """Delete duplicates

    [link=https://api.immich.app/endpoints/duplicates/deleteDuplicates]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["ids"], ids)
    bulk_ids_dto = BulkIdsDto.model_validate(json_data)
    kwargs["bulk_ids_dto"] = bulk_ids_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.duplicates, "delete_duplicates", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-asset-duplicates", deprecated=False, rich_help_panel="API commands")
def get_asset_duplicates(
    ctx: typer.Context,
) -> None:
    """Retrieve duplicates

    [link=https://api.immich.app/endpoints/duplicates/getAssetDuplicates]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.duplicates, "get_asset_duplicates", ctx, **kwargs
    )
    print_response(result, ctx)



"""Generated CLI commands for Users (admin) tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from typing import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""Administrative endpoints for managing users, including creating, updating, deleting, and restoring users. Also includes endpoints for resetting passwords and PIN codes.\n\n[link=https://api.immich.app/endpoints/users-admin]Immich API documentation[/link]"""
)


@app.command("create-user-admin", deprecated=False, rich_help_panel="API commands")
def create_user_admin(
    ctx: typer.Context,
    avatar_color: str | None = typer.Option(None, "--avatar-color", help=""""""),
    email: str = typer.Option(..., "--email", help=""""""),
    is_admin: Literal["true", "false"] | None = typer.Option(
        None, "--is-admin", help=""""""
    ),
    name: str = typer.Option(..., "--name", help=""""""),
    notify: Literal["true", "false"] | None = typer.Option(
        None, "--notify", help=""""""
    ),
    password: str = typer.Option(..., "--password", help=""""""),
    quota_size_in_bytes: int | None = typer.Option(
        None, "--quota-size-in-bytes", help="""""", min=0
    ),
    should_change_password: Literal["true", "false"] | None = typer.Option(
        None, "--should-change-password", help=""""""
    ),
    storage_label: str | None = typer.Option(None, "--storage-label", help=""""""),
) -> None:
    """Create a user

    [link=https://api.immich.app/endpoints/users-admin/createUserAdmin]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if avatar_color is not None:
        set_nested(json_data, ["avatar_color"], avatar_color)
    set_nested(json_data, ["email"], email)
    if is_admin is not None:
        set_nested(json_data, ["is_admin"], is_admin.lower() == "true")
    set_nested(json_data, ["name"], name)
    if notify is not None:
        set_nested(json_data, ["notify"], notify.lower() == "true")
    set_nested(json_data, ["password"], password)
    if quota_size_in_bytes is not None:
        set_nested(json_data, ["quota_size_in_bytes"], quota_size_in_bytes)
    if should_change_password is not None:
        set_nested(
            json_data,
            ["should_change_password"],
            should_change_password.lower() == "true",
        )
    if storage_label is not None:
        set_nested(json_data, ["storage_label"], storage_label)
    user_admin_create_dto = UserAdminCreateDto.model_validate(json_data)
    kwargs["user_admin_create_dto"] = user_admin_create_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users_admin, "create_user_admin", ctx, **kwargs)
    print_response(result, ctx)


@app.command("delete-user-admin", deprecated=False, rich_help_panel="API commands")
def delete_user_admin(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    force: Literal["true", "false"] | None = typer.Option(None, "--force", help=""""""),
) -> None:
    """Delete a user

    [link=https://api.immich.app/endpoints/users-admin/deleteUserAdmin]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    if force is not None:
        set_nested(json_data, ["force"], force.lower() == "true")
    user_admin_delete_dto = UserAdminDeleteDto.model_validate(json_data)
    kwargs["user_admin_delete_dto"] = user_admin_delete_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users_admin, "delete_user_admin", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-user-admin", deprecated=False, rich_help_panel="API commands")
def get_user_admin(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Retrieve a user

    [link=https://api.immich.app/endpoints/users-admin/getUserAdmin]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users_admin, "get_user_admin", ctx, **kwargs)
    print_response(result, ctx)


@app.command(
    "get-user-preferences-admin", deprecated=False, rich_help_panel="API commands"
)
def get_user_preferences_admin(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Retrieve user preferences

    [link=https://api.immich.app/endpoints/users-admin/getUserPreferencesAdmin]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.users_admin, "get_user_preferences_admin", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command(
    "get-user-sessions-admin", deprecated=False, rich_help_panel="API commands"
)
def get_user_sessions_admin(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Retrieve user sessions

    [link=https://api.immich.app/endpoints/users-admin/getUserSessionsAdmin]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.users_admin, "get_user_sessions_admin", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command(
    "get-user-statistics-admin", deprecated=False, rich_help_panel="API commands"
)
def get_user_statistics_admin(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    is_favorite: Literal["true", "false"] | None = typer.Option(
        None, "--is-favorite", help=""""""
    ),
    is_trashed: Literal["true", "false"] | None = typer.Option(
        None, "--is-trashed", help=""""""
    ),
    visibility: AssetVisibility | None = typer.Option(
        None, "--visibility", help=""""""
    ),
) -> None:
    """Retrieve user statistics

    [link=https://api.immich.app/endpoints/users-admin/getUserStatisticsAdmin]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    if is_favorite is not None:
        kwargs["is_favorite"] = is_favorite.lower() == "true"
    if is_trashed is not None:
        kwargs["is_trashed"] = is_trashed.lower() == "true"
    if visibility is not None:
        kwargs["visibility"] = visibility
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.users_admin, "get_user_statistics_admin", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("restore-user-admin", deprecated=False, rich_help_panel="API commands")
def restore_user_admin(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Restore a deleted user

    [link=https://api.immich.app/endpoints/users-admin/restoreUserAdmin]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.users_admin, "restore_user_admin", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("search-users-admin", deprecated=False, rich_help_panel="API commands")
def search_users_admin(
    ctx: typer.Context,
    id: str | None = typer.Option(None, "--id", help=""""""),
    with_deleted: Literal["true", "false"] | None = typer.Option(
        None, "--with-deleted", help=""""""
    ),
) -> None:
    """Search users

    [link=https://api.immich.app/endpoints/users-admin/searchUsersAdmin]Immich API documentation[/link]
    """
    kwargs = {}
    if id is not None:
        kwargs["id"] = id
    if with_deleted is not None:
        kwargs["with_deleted"] = with_deleted.lower() == "true"
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.users_admin, "search_users_admin", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("update-user-admin", deprecated=False, rich_help_panel="API commands")
def update_user_admin(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    avatar_color: str | None = typer.Option(None, "--avatar-color", help=""""""),
    email: str | None = typer.Option(None, "--email", help=""""""),
    is_admin: Literal["true", "false"] | None = typer.Option(
        None, "--is-admin", help=""""""
    ),
    name: str | None = typer.Option(None, "--name", help=""""""),
    password: str | None = typer.Option(None, "--password", help=""""""),
    pin_code: str | None = typer.Option(None, "--pin-code", help="""Example: 123456"""),
    quota_size_in_bytes: int | None = typer.Option(
        None, "--quota-size-in-bytes", help="""""", min=0
    ),
    should_change_password: Literal["true", "false"] | None = typer.Option(
        None, "--should-change-password", help=""""""
    ),
    storage_label: str | None = typer.Option(None, "--storage-label", help=""""""),
) -> None:
    """Update a user

    [link=https://api.immich.app/endpoints/users-admin/updateUserAdmin]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    if avatar_color is not None:
        set_nested(json_data, ["avatar_color"], avatar_color)
    if email is not None:
        set_nested(json_data, ["email"], email)
    if is_admin is not None:
        set_nested(json_data, ["is_admin"], is_admin.lower() == "true")
    if name is not None:
        set_nested(json_data, ["name"], name)
    if password is not None:
        set_nested(json_data, ["password"], password)
    if pin_code is not None:
        set_nested(json_data, ["pin_code"], pin_code)
    if quota_size_in_bytes is not None:
        set_nested(json_data, ["quota_size_in_bytes"], quota_size_in_bytes)
    if should_change_password is not None:
        set_nested(
            json_data,
            ["should_change_password"],
            should_change_password.lower() == "true",
        )
    if storage_label is not None:
        set_nested(json_data, ["storage_label"], storage_label)
    user_admin_update_dto = UserAdminUpdateDto.model_validate(json_data)
    kwargs["user_admin_update_dto"] = user_admin_update_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.users_admin, "update_user_admin", ctx, **kwargs)
    print_response(result, ctx)


@app.command(
    "update-user-preferences-admin", deprecated=False, rich_help_panel="API commands"
)
def update_user_preferences_admin(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    albums_default_asset_order: str | None = typer.Option(
        None, "--albums-default-asset-order", help=""""""
    ),
    avatar_color: str | None = typer.Option(None, "--avatar-color", help=""""""),
    cast_g_cast_enabled: Literal["true", "false"] | None = typer.Option(
        None, "--cast-g-cast-enabled", help=""""""
    ),
    download_archive_size: int | None = typer.Option(
        None, "--download-archive-size", help="""""", min=1
    ),
    download_include_embedded_videos: Literal["true", "false"] | None = typer.Option(
        None, "--download-include-embedded-videos", help=""""""
    ),
    email_notifications_album_invite: Literal["true", "false"] | None = typer.Option(
        None, "--email-notifications-album-invite", help=""""""
    ),
    email_notifications_album_update: Literal["true", "false"] | None = typer.Option(
        None, "--email-notifications-album-update", help=""""""
    ),
    email_notifications_enabled: Literal["true", "false"] | None = typer.Option(
        None, "--email-notifications-enabled", help=""""""
    ),
    folders_enabled: Literal["true", "false"] | None = typer.Option(
        None, "--folders-enabled", help=""""""
    ),
    folders_sidebar_web: Literal["true", "false"] | None = typer.Option(
        None, "--folders-sidebar-web", help=""""""
    ),
    memories_duration: int | None = typer.Option(
        None, "--memories-duration", help="""""", min=1
    ),
    memories_enabled: Literal["true", "false"] | None = typer.Option(
        None, "--memories-enabled", help=""""""
    ),
    people_enabled: Literal["true", "false"] | None = typer.Option(
        None, "--people-enabled", help=""""""
    ),
    people_sidebar_web: Literal["true", "false"] | None = typer.Option(
        None, "--people-sidebar-web", help=""""""
    ),
    purchase_hide_buy_button_until: str | None = typer.Option(
        None, "--purchase-hide-buy-button-until", help=""""""
    ),
    purchase_show_support_badge: Literal["true", "false"] | None = typer.Option(
        None, "--purchase-show-support-badge", help=""""""
    ),
    ratings_enabled: Literal["true", "false"] | None = typer.Option(
        None, "--ratings-enabled", help=""""""
    ),
    shared_links_enabled: Literal["true", "false"] | None = typer.Option(
        None, "--shared-links-enabled", help=""""""
    ),
    shared_links_sidebar_web: Literal["true", "false"] | None = typer.Option(
        None, "--shared-links-sidebar-web", help=""""""
    ),
    tags_enabled: Literal["true", "false"] | None = typer.Option(
        None, "--tags-enabled", help=""""""
    ),
    tags_sidebar_web: Literal["true", "false"] | None = typer.Option(
        None, "--tags-sidebar-web", help=""""""
    ),
) -> None:
    """Update user preferences

    [link=https://api.immich.app/endpoints/users-admin/updateUserPreferencesAdmin]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    if albums_default_asset_order is not None:
        set_nested(
            json_data, ["albums_default_asset_order"], albums_default_asset_order
        )
    if avatar_color is not None:
        set_nested(json_data, ["avatar_color"], avatar_color)
    if cast_g_cast_enabled is not None:
        set_nested(
            json_data, ["cast_g_cast_enabled"], cast_g_cast_enabled.lower() == "true"
        )
    if download_archive_size is not None:
        set_nested(json_data, ["download_archive_size"], download_archive_size)
    if download_include_embedded_videos is not None:
        set_nested(
            json_data,
            ["download_include_embedded_videos"],
            download_include_embedded_videos.lower() == "true",
        )
    if email_notifications_album_invite is not None:
        set_nested(
            json_data,
            ["email_notifications_album_invite"],
            email_notifications_album_invite.lower() == "true",
        )
    if email_notifications_album_update is not None:
        set_nested(
            json_data,
            ["email_notifications_album_update"],
            email_notifications_album_update.lower() == "true",
        )
    if email_notifications_enabled is not None:
        set_nested(
            json_data,
            ["email_notifications_enabled"],
            email_notifications_enabled.lower() == "true",
        )
    if folders_enabled is not None:
        set_nested(json_data, ["folders_enabled"], folders_enabled.lower() == "true")
    if folders_sidebar_web is not None:
        set_nested(
            json_data, ["folders_sidebar_web"], folders_sidebar_web.lower() == "true"
        )
    if memories_duration is not None:
        set_nested(json_data, ["memories_duration"], memories_duration)
    if memories_enabled is not None:
        set_nested(json_data, ["memories_enabled"], memories_enabled.lower() == "true")
    if people_enabled is not None:
        set_nested(json_data, ["people_enabled"], people_enabled.lower() == "true")
    if people_sidebar_web is not None:
        set_nested(
            json_data, ["people_sidebar_web"], people_sidebar_web.lower() == "true"
        )
    if purchase_hide_buy_button_until is not None:
        set_nested(
            json_data,
            ["purchase_hide_buy_button_until"],
            purchase_hide_buy_button_until,
        )
    if purchase_show_support_badge is not None:
        set_nested(
            json_data,
            ["purchase_show_support_badge"],
            purchase_show_support_badge.lower() == "true",
        )
    if ratings_enabled is not None:
        set_nested(json_data, ["ratings_enabled"], ratings_enabled.lower() == "true")
    if shared_links_enabled is not None:
        set_nested(
            json_data, ["shared_links_enabled"], shared_links_enabled.lower() == "true"
        )
    if shared_links_sidebar_web is not None:
        set_nested(
            json_data,
            ["shared_links_sidebar_web"],
            shared_links_sidebar_web.lower() == "true",
        )
    if tags_enabled is not None:
        set_nested(json_data, ["tags_enabled"], tags_enabled.lower() == "true")
    if tags_sidebar_web is not None:
        set_nested(json_data, ["tags_sidebar_web"], tags_sidebar_web.lower() == "true")
    user_preferences_update_dto = UserPreferencesUpdateDto.model_validate(json_data)
    kwargs["user_preferences_update_dto"] = user_preferences_update_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.users_admin, "update_user_preferences_admin", ctx, **kwargs
    )
    print_response(result, ctx)



from typing import Any, Optional, cast, overload
import json

from rich import print as print_rich, print_json
from rich.table import Table
import rtoml
import typer

from immich.cli.consts import CONFIG_FILE, DEFAULT_FORMAT, SECRET_KEYS
from immich.cli.types import ClientConfig, FormatMode, PrintType


def set_path(data: dict[str, Any], path: str, value: Any) -> None:
    """
    Set a nested dictionary value using a path list.

    Example: set_path({}, 'user.name', 'John') -> {'user': {'name': 'John'}}
    """
    parts = path.split(".")
    cur = data
    for p in parts[:-1]:
        cur = cur.setdefault(p, {})
    cur[parts[-1]] = value


def get_path(data: dict[str, Any], path: str) -> Any:
    """
    Get a nested dictionary value using a path list.
    """
    parts = path.split(".")
    cur = data
    for p in parts:
        cur = cur[p]
    return cur


def load_config() -> dict[str, Any]:
    """
    Load the config file. Returns an empty dict if the file does not exist.
    """
    if not CONFIG_FILE.exists():
        return {}
    return rtoml.load(CONFIG_FILE, none_value="")


def write_config(data: dict[str, Any]) -> None:
    """
    Write the config data to the config file and set permissions.
    """
    CONFIG_FILE.parent.mkdir(parents=True, exist_ok=True)
    with CONFIG_FILE.open("w") as f:
        rtoml.dump(data, f, none_value="")
    CONFIG_FILE.chmod(0o600)


def check_config() -> None:
    """
    Check if the config file exists.
    """
    if not CONFIG_FILE.exists():
        print_(
            "Config file does not exist. Run [bold]immich config set[/bold] to create it.",
            type="error",
        )
        raise typer.Exit(code=1)


def resolve_client_config(
    config: ClientConfig, profile: str, profile_explicit: bool = False
) -> ClientConfig:
    """
    Resolve the client config from the config file.

    :param config: The config to resolve
    :param profile: The profile to use
    :param profile_explicit: Whether the profile was explicitly set by the user
    :raises typer.Exit: If the profile is not found in the config
    :return: The resolved config
    """
    data = load_config()
    profiles: dict[str, Any] = data.get("profiles", {})

    if profile_explicit and profile not in profiles:
        print_(
            f"Profile '{profile}' not found. Run immich config set --profile {profile}.",
            type="error",
        )
        raise typer.Exit(code=1)

    profile_data = profiles.get(profile, {})

    return ClientConfig(
        api_key=config.api_key or profile_data.get("api_key"),
        access_token=config.access_token or profile_data.get("access_token"),
        base_url=config.base_url or profile_data.get("base_url"),
    )


def _is_secret_key(key: str) -> bool:
    """Check if a key indicates a secret value."""
    return any(secret in key.lower() for secret in SECRET_KEYS)


def _redact_secret(secret: str, start: int = 3, end: int = 3) -> str:
    """
    Redact a secret by showing only the first `start` and last `end` characters.

    :param secret: The secret string to redact
    :param start: Number of characters to show at the start (default: 3)
    :param end: Number of characters to show at the end (default: 3)
    :return: The redacted secret string
    """
    if not secret:
        return ""

    length = len(secret)
    if length <= start + end:
        return "*" * length

    return secret[:start] + "*" * (length - start - end) + secret[-end:]


@overload
def mask(secret: str, start: int = 3, end: int = 3) -> str: ...


@overload
def mask(obj: Any, start: int = 3, end: int = 3, key: Optional[str] = None) -> Any: ...


def mask(obj: Any, start: int = 3, end: int = 3, key: Optional[str] = None) -> Any:
    """
    Recursively mask secret values in nested dictionaries and lists.

    :param obj: The object to mask (dict, list, or any other type)
    :param start: Number of characters to show at the start (default: 3)
    :param end: Number of characters to show at the end (default: 3)
    :param key: Optional key path for top-level values to check if they're secrets
    :return: The masked object with secrets redacted using _redact_secret
    """
    if isinstance(obj, dict):
        return {
            k: (
                _redact_secret(v, start, end)
                if _is_secret_key(k) and isinstance(v, str)
                else mask(v, start, end, k)
            )
            for k, v in obj.items()
        }
    if isinstance(obj, list):
        return [mask(v, start, end) for v in obj]
    if isinstance(obj, str):
        if key is None or _is_secret_key(key):
            return _redact_secret(obj, start, end)
    return obj


def _print_table(data: Any) -> None:  # pragma: no cover
    """
    Print data as a table using rich.
    For dicts: creates a table with key/value columns.
    For lists: prints multiple tables.
    """
    if isinstance(data, list):
        for item in data:
            _print_table(item)
    elif isinstance(data, dict):
        _print_dict_table(data)
    else:
        print_rich(str(data))


def _print_dict_table(data: dict[str, Any]) -> None:  # pragma: no cover
    """Print a dictionary as a table with key and value columns."""
    table = Table(show_header=True, header_style="bold")
    table.add_column("Key", style="cyan")
    table.add_column("Value", style="green")

    for key, value in data.items():
        table.add_row(key, str(value))

    print_rich(table)


def print_(
    message: str,
    *,
    type: PrintType,
    ctx: Optional[typer.Context] = None,
) -> None:
    """
    Print a message in the given format as a rich print or a plain print.
    :param message: The message to print
    :param type: The type of the message
    :param ctx: The context to use
    """
    match type:
        case "json":
            try:
                format_mode = cast(FormatMode, ctx.obj.get("format", DEFAULT_FORMAT))  # type: ignore[possibly-missing-attribute]
            except (AttributeError, KeyError):
                format_mode = DEFAULT_FORMAT
            match format_mode:
                case "pretty":
                    print_json(message)
                case "json":
                    print(message)
                case "table":  # pragma: no cover
                    try:
                        data = json.loads(message)
                        _print_table(data)
                    except json.JSONDecodeError:
                        print(message)
                    print_("The 'table' format is experimental.", type="warning")
        case "text":
            print(message)
        case "info":
            print_rich(message)
        case "warning":
            print_rich(f"[yellow][bold][Warning][/bold] {message}[/yellow]")
        case "error":
            print_rich(f"[red][bold][Error][/bold] {message}[/red]")
        case "success":
            print_rich(f"[green][bold][Success][/bold][/green] {message}")
        case "debug":
            if ctx is not None and ctx.obj["verbose"]:
                print_rich(f"[blue][bold][Debug][/bold] {message}[/blue]")



"""Generated CLI commands for Authentication (admin) tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command
from immich.client.generated.models import *

app = typer.Typer(
    help="""Administrative endpoints related to authentication.\n\n[link=https://api.immich.app/endpoints/authentication-admin]Immich API documentation[/link]"""
)


@app.command(
    "unlink-all-o-auth-accounts-admin", deprecated=False, rich_help_panel="API commands"
)
def unlink_all_o_auth_accounts_admin(
    ctx: typer.Context,
) -> None:
    """Unlink all OAuth accounts

    [link=https://api.immich.app/endpoints/authentication-admin/unlinkAllOAuthAccountsAdmin]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.auth_admin, "unlink_all_o_auth_accounts_admin", ctx, **kwargs
    )
    print_response(result, ctx)



"""Generated CLI commands for Faces tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""A face is a detected human face within an asset, which can be associated with a person. Faces are normally detected via machine learning, but can also be created via manually.\n\n[link=https://api.immich.app/endpoints/faces]Immich API documentation[/link]"""
)


@app.command("create-face", deprecated=False, rich_help_panel="API commands")
def create_face(
    ctx: typer.Context,
    asset_id: str = typer.Option(..., "--asset-id", help=""""""),
    height: int = typer.Option(..., "--height", help=""""""),
    image_height: int = typer.Option(..., "--image-height", help=""""""),
    image_width: int = typer.Option(..., "--image-width", help=""""""),
    person_id: str = typer.Option(..., "--person-id", help=""""""),
    width: int = typer.Option(..., "--width", help=""""""),
    x: int = typer.Option(..., "--x", help=""""""),
    y: int = typer.Option(..., "--y", help=""""""),
) -> None:
    """Create a face

    [link=https://api.immich.app/endpoints/faces/createFace]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["asset_id"], asset_id)
    set_nested(json_data, ["height"], height)
    set_nested(json_data, ["image_height"], image_height)
    set_nested(json_data, ["image_width"], image_width)
    set_nested(json_data, ["person_id"], person_id)
    set_nested(json_data, ["width"], width)
    set_nested(json_data, ["x"], x)
    set_nested(json_data, ["y"], y)
    asset_face_create_dto = AssetFaceCreateDto.model_validate(json_data)
    kwargs["asset_face_create_dto"] = asset_face_create_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.faces, "create_face", ctx, **kwargs)
    print_response(result, ctx)


@app.command("delete-face", deprecated=False, rich_help_panel="API commands")
def delete_face(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    force: bool = typer.Option(..., "--force", help=""""""),
) -> None:
    """Delete a face

    [link=https://api.immich.app/endpoints/faces/deleteFace]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    set_nested(json_data, ["force"], force)
    asset_face_delete_dto = AssetFaceDeleteDto.model_validate(json_data)
    kwargs["asset_face_delete_dto"] = asset_face_delete_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.faces, "delete_face", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-faces", deprecated=False, rich_help_panel="API commands")
def get_faces(
    ctx: typer.Context,
    id: str = typer.Option(..., "--id", help=""""""),
) -> None:
    """Retrieve faces for asset

    [link=https://api.immich.app/endpoints/faces/getFaces]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.faces, "get_faces", ctx, **kwargs)
    print_response(result, ctx)


@app.command("reassign-faces-by-id", deprecated=False, rich_help_panel="API commands")
def reassign_faces_by_id(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    body_id: str = typer.Option(..., "--body-id", help=""""""),
) -> None:
    """Re-assign a face to another person

    [link=https://api.immich.app/endpoints/faces/reassignFacesById]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    set_nested(json_data, ["body_id"], body_id)
    face_dto = FaceDto.model_validate(json_data)
    kwargs["face_dto"] = face_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.faces, "reassign_faces_by_id", ctx, **kwargs)
    print_response(result, ctx)



"""Generated CLI commands for Notifications tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from datetime import datetime
from typing import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""A notification is a specialized message sent to users to inform them of important events. Currently, these notifications are only shown in the Immich web application.\n\n[link=https://api.immich.app/endpoints/notifications]Immich API documentation[/link]"""
)


@app.command("delete-notification", deprecated=False, rich_help_panel="API commands")
def delete_notification(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Delete a notification

    [link=https://api.immich.app/endpoints/notifications/deleteNotification]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.notifications, "delete_notification", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("delete-notifications", deprecated=False, rich_help_panel="API commands")
def delete_notifications(
    ctx: typer.Context,
    ids: list[str] = typer.Option(..., "--ids", help=""""""),
) -> None:
    """Delete notifications

    [link=https://api.immich.app/endpoints/notifications/deleteNotifications]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["ids"], ids)
    notification_delete_all_dto = NotificationDeleteAllDto.model_validate(json_data)
    kwargs["notification_delete_all_dto"] = notification_delete_all_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.notifications, "delete_notifications", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("get-notification", deprecated=False, rich_help_panel="API commands")
def get_notification(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Get a notification

    [link=https://api.immich.app/endpoints/notifications/getNotification]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.notifications, "get_notification", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("get-notifications", deprecated=False, rich_help_panel="API commands")
def get_notifications(
    ctx: typer.Context,
    id: str | None = typer.Option(None, "--id", help=""""""),
    level: NotificationLevel | None = typer.Option(None, "--level", help=""""""),
    type: NotificationType | None = typer.Option(None, "--type", help=""""""),
    unread: Literal["true", "false"] | None = typer.Option(
        None, "--unread", help=""""""
    ),
) -> None:
    """Retrieve notifications

    [link=https://api.immich.app/endpoints/notifications/getNotifications]Immich API documentation[/link]
    """
    kwargs = {}
    if id is not None:
        kwargs["id"] = id
    if level is not None:
        kwargs["level"] = level
    if type is not None:
        kwargs["type"] = type
    if unread is not None:
        kwargs["unread"] = unread.lower() == "true"
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.notifications, "get_notifications", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("update-notification", deprecated=False, rich_help_panel="API commands")
def update_notification(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    read_at: datetime | None = typer.Option(None, "--read-at", help=""""""),
) -> None:
    """Update a notification

    [link=https://api.immich.app/endpoints/notifications/updateNotification]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    if read_at is not None:
        set_nested(json_data, ["read_at"], read_at)
    notification_update_dto = NotificationUpdateDto.model_validate(json_data)
    kwargs["notification_update_dto"] = notification_update_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.notifications, "update_notification", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command("update-notifications", deprecated=False, rich_help_panel="API commands")
def update_notifications(
    ctx: typer.Context,
    ids: list[str] = typer.Option(..., "--ids", help=""""""),
    read_at: datetime | None = typer.Option(None, "--read-at", help=""""""),
) -> None:
    """Update notifications

    [link=https://api.immich.app/endpoints/notifications/updateNotifications]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["ids"], ids)
    if read_at is not None:
        set_nested(json_data, ["read_at"], read_at)
    notification_update_all_dto = NotificationUpdateAllDto.model_validate(json_data)
    kwargs["notification_update_all_dto"] = notification_update_all_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.notifications, "update_notifications", ctx, **kwargs
    )
    print_response(result, ctx)



"""Generated CLI commands for Jobs tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from typing import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""Queues and background jobs are used for processing tasks asynchronously. Queues can be paused and resumed as needed.\n\n[link=https://api.immich.app/endpoints/jobs]Immich API documentation[/link]"""
)


@app.command("create-job", deprecated=False, rich_help_panel="API commands")
def create_job(
    ctx: typer.Context,
    name: str = typer.Option(..., "--name", help=""""""),
) -> None:
    """Create a manual job

    [link=https://api.immich.app/endpoints/jobs/createJob]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["name"], name)
    job_create_dto = JobCreateDto.model_validate(json_data)
    kwargs["job_create_dto"] = job_create_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.jobs, "create_job", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-queues-legacy", deprecated=True, rich_help_panel="API commands")
def get_queues_legacy(
    ctx: typer.Context,
) -> None:
    """Retrieve queue counts and status

    [link=https://api.immich.app/endpoints/jobs/getQueuesLegacy]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.jobs, "get_queues_legacy", ctx, **kwargs)
    print_response(result, ctx)


@app.command(
    "run-queue-command-legacy", deprecated=True, rich_help_panel="API commands"
)
def run_queue_command_legacy(
    ctx: typer.Context,
    name: QueueName = typer.Argument(..., help=""""""),
    command: str = typer.Option(..., "--command", help=""""""),
    force: Literal["true", "false"] | None = typer.Option(None, "--force", help=""""""),
) -> None:
    """Run jobs

    [link=https://api.immich.app/endpoints/jobs/runQueueCommandLegacy]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["name"] = name
    set_nested(json_data, ["command"], command)
    if force is not None:
        set_nested(json_data, ["force"], force.lower() == "true")
    queue_command_dto = QueueCommandDto.model_validate(json_data)
    kwargs["queue_command_dto"] = queue_command_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.jobs, "run_queue_command_legacy", ctx, **kwargs)
    print_response(result, ctx)



"""Generated CLI commands for Views tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command
from immich.client.generated.models import *

app = typer.Typer(
    help="""Endpoints for specialized views, such as the folder view.\n\n[link=https://api.immich.app/endpoints/views]Immich API documentation[/link]"""
)


@app.command(
    "get-assets-by-original-path", deprecated=False, rich_help_panel="API commands"
)
def get_assets_by_original_path(
    ctx: typer.Context,
    path: str = typer.Option(..., "--path", help=""""""),
) -> None:
    """Retrieve assets by original path

    [link=https://api.immich.app/endpoints/views/getAssetsByOriginalPath]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["path"] = path
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.views, "get_assets_by_original_path", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command(
    "get-unique-original-paths", deprecated=False, rich_help_panel="API commands"
)
def get_unique_original_paths(
    ctx: typer.Context,
) -> None:
    """Retrieve unique paths

    [link=https://api.immich.app/endpoints/views/getUniqueOriginalPaths]Immich API documentation[/link]
    """
    kwargs = {}
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.views, "get_unique_original_paths", ctx, **kwargs
    )
    print_response(result, ctx)



"""Generated CLI commands for Notifications (admin) tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
import json
from datetime import datetime
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""Notification administrative endpoints.\n\n[link=https://api.immich.app/endpoints/notifications-admin]Immich API documentation[/link]"""
)


@app.command("create-notification", deprecated=False, rich_help_panel="API commands")
def create_notification(
    ctx: typer.Context,
    data: str | None = typer.Option(None, "--data", help="""As a JSON string"""),
    description: str | None = typer.Option(None, "--description", help=""""""),
    level: str | None = typer.Option(None, "--level", help=""""""),
    read_at: datetime | None = typer.Option(None, "--read-at", help=""""""),
    title: str = typer.Option(..., "--title", help=""""""),
    type: str | None = typer.Option(None, "--type", help=""""""),
    user_id: str = typer.Option(..., "--user-id", help=""""""),
) -> None:
    """Create a notification

    [link=https://api.immich.app/endpoints/notifications-admin/createNotification]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if data is not None:
        value_data = [json.loads(i) for i in data]
        set_nested(json_data, ["data"], value_data)
    if description is not None:
        set_nested(json_data, ["description"], description)
    if level is not None:
        set_nested(json_data, ["level"], level)
    if read_at is not None:
        set_nested(json_data, ["read_at"], read_at)
    set_nested(json_data, ["title"], title)
    if type is not None:
        set_nested(json_data, ["type"], type)
    set_nested(json_data, ["user_id"], user_id)
    notification_create_dto = NotificationCreateDto.model_validate(json_data)
    kwargs["notification_create_dto"] = notification_create_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.notifications_admin, "create_notification", ctx, **kwargs
    )
    print_response(result, ctx)


@app.command(
    "get-notification-template-admin", deprecated=False, rich_help_panel="API commands"
)
def get_notification_template_admin(
    ctx: typer.Context,
    name: str = typer.Argument(..., help=""""""),
    template: str = typer.Option(..., "--template", help=""""""),
) -> None:
    """Render email template

    [link=https://api.immich.app/endpoints/notifications-admin/getNotificationTemplateAdmin]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["name"] = name
    set_nested(json_data, ["template"], template)
    template_dto = TemplateDto.model_validate(json_data)
    kwargs["template_dto"] = template_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client,
        client.notifications_admin,
        "get_notification_template_admin",
        ctx,
        **kwargs,
    )
    print_response(result, ctx)


@app.command("send-test-email-admin", deprecated=False, rich_help_panel="API commands")
def send_test_email_admin(
    ctx: typer.Context,
    enabled: bool = typer.Option(..., "--enabled", help=""""""),
    from_: str = typer.Option(..., "--from", help=""""""),
    reply_to: str = typer.Option(..., "--reply-to", help=""""""),
    transport_host: str = typer.Option(..., "--transport-host", help=""""""),
    transport_ignore_cert: bool = typer.Option(
        ..., "--transport-ignore-cert", help=""""""
    ),
    transport_password: str = typer.Option(..., "--transport-password", help=""""""),
    transport_port: float = typer.Option(
        ..., "--transport-port", help="""""", min=0, max=65535
    ),
    transport_secure: bool = typer.Option(..., "--transport-secure", help=""""""),
    transport_username: str = typer.Option(..., "--transport-username", help=""""""),
) -> None:
    """Send test email

    [link=https://api.immich.app/endpoints/notifications-admin/sendTestEmailAdmin]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["enabled"], enabled)
    set_nested(json_data, ["from_"], from_)
    set_nested(json_data, ["reply_to"], reply_to)
    set_nested(json_data, ["transport_host"], transport_host)
    set_nested(json_data, ["transport_ignore_cert"], transport_ignore_cert)
    set_nested(json_data, ["transport_password"], transport_password)
    set_nested(json_data, ["transport_port"], transport_port)
    set_nested(json_data, ["transport_secure"], transport_secure)
    set_nested(json_data, ["transport_username"], transport_username)
    system_config_smtp_dto = SystemConfigSmtpDto.model_validate(json_data)
    kwargs["system_config_smtp_dto"] = system_config_smtp_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.notifications_admin, "send_test_email_admin", ctx, **kwargs
    )
    print_response(result, ctx)



"""CLI wrapper commands for AssetsApiWrapped convenience methods."""

from __future__ import annotations

from pathlib import Path

import typer

from typing import TYPE_CHECKING

from immich.cli.commands import assets as assets_commands
from immich.cli.runtime import print_response, run_command

if TYPE_CHECKING:
    from immich.client.main import AsyncClient
# Reuse the existing app from the generated commands
app = assets_commands.app


@app.command("download-asset-to-file", rich_help_panel="Custom commands")
def download_asset_to_file(
    ctx: typer.Context,
    id: str = typer.Argument(..., help="Asset ID (UUID)"),
    out_dir: Path = typer.Argument(
        ..., help="Output directory for the downloaded file"
    ),
    key: str | None = typer.Option(
        None, "--key", help="Public share key (last path segment of /share/<key>)"
    ),
    slug: str | None = typer.Option(
        None, "--slug", help="Public share slug (last path segment of /s/<slug>)"
    ),
    filename: str | None = typer.Option(
        None,
        "--filename",
        help="Filename to use (defaults to original filename or orig-{asset_id})",
    ),
    show_progress: bool = typer.Option(
        False,
        "--show-progress",
        help="Show progress bar while downloading",
    ),
) -> None:  # pragma: no cover
    """Download an asset to a file.

    Downloads the original asset file and saves it to the specified output directory.
    The filename can be specified or will be derived from the response headers.
    """
    kwargs = {}
    kwargs["id"] = id
    kwargs["out_dir"] = out_dir
    kwargs["show_progress"] = show_progress
    if key is not None:
        kwargs["key"] = key
    if slug is not None:
        kwargs["slug"] = slug
    if filename is not None:
        kwargs["filename"] = filename
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "download_asset_to_file", **kwargs)
    print_response(result, ctx)


@app.command("play-asset-video-to-file", rich_help_panel="Custom commands")
def play_asset_video_to_file(
    ctx: typer.Context,
    id: str = typer.Argument(..., help="Asset ID (UUID)"),
    out_dir: Path = typer.Argument(..., help="Output directory for the video file"),
    key: str | None = typer.Option(
        None, "--key", help="Public share key (last path segment of /share/<key>)"
    ),
    slug: str | None = typer.Option(
        None, "--slug", help="Public share slug (last path segment of /s/<slug>)"
    ),
    filename: str | None = typer.Option(
        None,
        "--filename",
        help="Filename to use (defaults to original filename or video-{asset_id})",
    ),
    show_progress: bool = typer.Option(
        False,
        "--show-progress",
        help="Show progress bar while downloading",
    ),
) -> None:  # pragma: no cover
    """Save an asset's video stream to a file.

    Downloads the video stream for the asset and saves it to the specified output directory.
    The filename can be specified or will be derived from the response headers.
    """
    kwargs = {}
    kwargs["id"] = id
    kwargs["out_dir"] = out_dir
    kwargs["show_progress"] = show_progress
    if key is not None:
        kwargs["key"] = key
    if slug is not None:
        kwargs["slug"] = slug
    if filename is not None:
        kwargs["filename"] = filename
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "play_asset_video_to_file", **kwargs)
    print_response(result, ctx)


@app.command("view-asset-to-file", rich_help_panel="Custom commands")
def view_asset_to_file(
    ctx: typer.Context,
    id: str = typer.Argument(..., help="Asset ID (UUID)"),
    out_dir: Path = typer.Argument(..., help="Output directory for the thumbnail file"),
    key: str | None = typer.Option(
        None, "--key", help="Public share key (last path segment of /share/<key>)"
    ),
    slug: str | None = typer.Option(
        None, "--slug", help="Public share slug (last path segment of /s/<slug>)"
    ),
    size: str | None = typer.Option(
        None, "--size", help="Thumbnail size: fullsize, preview, or thumbnail"
    ),
    filename: str | None = typer.Option(
        None,
        "--filename",
        help="Filename to use (defaults to original filename or thumb-{asset_id})",
    ),
    show_progress: bool = typer.Option(
        False,
        "--show-progress",
        help="Show progress bar while downloading",
    ),
) -> None:  # pragma: no cover
    """Save an asset's thumbnail to a file.

    Downloads the thumbnail for the asset and saves it to the specified output directory.
    The filename can be specified or will be derived from the response headers.
    """
    kwargs = {}
    kwargs["id"] = id
    kwargs["out_dir"] = out_dir
    kwargs["show_progress"] = show_progress
    if key is not None:
        kwargs["key"] = key
    if slug is not None:
        kwargs["slug"] = slug
    if size is not None:
        kwargs["size"] = size
    if filename is not None:
        kwargs["filename"] = filename
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "view_asset_to_file", **kwargs)
    print_response(result, ctx)


@app.command("upload", rich_help_panel="Custom commands")
def upload(
    ctx: typer.Context,
    paths: list[Path] = typer.Argument(
        ...,
        help="File or directory paths to upload (can specify multiple)",
        exists=True,
    ),
    ignore_pattern: str | None = typer.Option(
        None,
        "--ignore-pattern",
        help="Wildcard pattern to ignore files (uses fnmatch, not regex)",
    ),
    include_hidden: bool = typer.Option(
        False,
        "--include-hidden",
        help="Include hidden files (starting with '.')",
    ),
    skip_duplicates: bool = typer.Option(
        False,
        "--skip-duplicates",
        help="Check for duplicates using SHA1 hashes before uploading",
    ),
    concurrency: int = typer.Option(
        5, "--concurrency", help="Number of concurrent uploads"
    ),
    show_progress: bool = typer.Option(
        False, "--show-progress", help="Show progress bars"
    ),
    album_name: str | None = typer.Option(
        None,
        "--album-name",
        help="Album name to create or use (if not provided, no album operations are performed)",
    ),
    delete_uploads: bool = typer.Option(
        False,
        "--delete-uploads",
        help="Delete successfully uploaded files locally",
    ),
    delete_duplicates: bool = typer.Option(
        False,
        "--delete-duplicates",
        help="Delete rejected duplicate files locally",
    ),
    dry_run: bool = typer.Option(
        False,
        "--dry-run",
        help="Simulate uploads without actually uploading",
    ),
) -> None:  # pragma: no cover
    """Upload assets with smart features.

    Uploads files or directories with duplicate detection, album management, sidecar support, and dry run capability.
    Directories are automatically walked recursively.
    """
    kwargs = {}
    kwargs["paths"] = paths
    if ignore_pattern is not None:
        kwargs["ignore_pattern"] = ignore_pattern
    if album_name is not None:
        kwargs["album_name"] = album_name
    kwargs["include_hidden"] = include_hidden
    kwargs["skip_duplicates"] = skip_duplicates
    kwargs["concurrency"] = concurrency
    kwargs["show_progress"] = show_progress
    kwargs["delete_uploads"] = delete_uploads
    kwargs["delete_duplicates"] = delete_duplicates
    kwargs["dry_run"] = dry_run
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.assets, "upload", **kwargs)
    print_response(result, ctx)



"""Runtime helpers for executing async client calls and handling output."""

from __future__ import annotations

import asyncio
import json
import traceback
from typing import Any, Awaitable, Callable

from immich.cli.utils import print_
from pydantic import BaseModel
from typer import Context, Exit

from immich.cli.types import MaybeBaseModel

from immich import AsyncClient
from immich.client.generated.exceptions import ApiException


def set_nested(d: dict[str, Any], path: list[str], value: Any) -> None:
    """Set a nested dictionary value using a path list.

    Example: set_nested({}, ['user', 'name'], 'John') -> {'user': {'name': 'John'}}
    """
    current = d
    for part in path[:-1]:
        if part not in current:
            current[part] = {}
        elif not isinstance(current[part], dict):
            current[part] = {}
        current = current[part]
    current[path[-1]] = value


def print_response(data: MaybeBaseModel, ctx: Context) -> None:
    """Print response data."""

    def convert_to_dict(obj: MaybeBaseModel) -> Any:
        """Recursively convert Pydantic models to dicts."""
        if isinstance(obj, list):
            return [convert_to_dict(item) for item in obj]
        elif isinstance(obj, BaseModel):
            return obj.model_dump()
        else:
            return obj

    json_str = json.dumps(convert_to_dict(data), default=str)

    print_(message=json_str, type="json", ctx=ctx)


def format_api_error(e: ApiException) -> tuple[str, int]:
    """Return (message, exit_code) for an ApiException."""
    exit_code = 1 if e.status is None else e.status // 100

    if not e.body:
        return ("API error", exit_code)

    if isinstance(e.body, str):
        return (e.body, exit_code)

    return (json.dumps(e.body, default=str), exit_code)


async def run_async(coro: Awaitable[Any]) -> Any:
    return await coro


def run_command(
    client: AsyncClient,
    api_group: Any,
    method_name: str,
    ctx: Context | None = None,
    **kwargs: Any,
) -> Any:
    """Run a client API method and handle the result."""
    method: Callable[..., Awaitable[Any]] = getattr(api_group, method_name)

    async def _call_and_close() -> Any:
        try:
            return await method(**kwargs)
        finally:
            await client.close()

    try:
        return asyncio.run(_call_and_close())

    except ApiException as e:
        message, code = format_api_error(e)
        print_(message, type="error", ctx=ctx)
        print_(traceback.format_exc(), type="debug", ctx=ctx)
        raise Exit(code=code)

    except Exception as e:
        print_(f"Unexpected error: {str(e).strip()}", type="error", ctx=ctx)
        print_(traceback.format_exc(), type="debug", ctx=ctx)
        raise Exit(code=1)



"""Generated CLI commands for Activities tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""An activity is a like or a comment made by a user on an asset or album.\n\n[link=https://api.immich.app/endpoints/activities]Immich API documentation[/link]"""
)


@app.command("create-activity", deprecated=False, rich_help_panel="API commands")
def create_activity(
    ctx: typer.Context,
    album_id: str = typer.Option(..., "--album-id", help=""""""),
    asset_id: str | None = typer.Option(None, "--asset-id", help=""""""),
    comment: str | None = typer.Option(None, "--comment", help=""""""),
    type: str = typer.Option(..., "--type", help=""""""),
) -> None:
    """Create an activity

    [link=https://api.immich.app/endpoints/activities/createActivity]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["album_id"], album_id)
    if asset_id is not None:
        set_nested(json_data, ["asset_id"], asset_id)
    if comment is not None:
        set_nested(json_data, ["comment"], comment)
    set_nested(json_data, ["type"], type)
    activity_create_dto = ActivityCreateDto.model_validate(json_data)
    kwargs["activity_create_dto"] = activity_create_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.activities, "create_activity", ctx, **kwargs)
    print_response(result, ctx)


@app.command("delete-activity", deprecated=False, rich_help_panel="API commands")
def delete_activity(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Delete an activity

    [link=https://api.immich.app/endpoints/activities/deleteActivity]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.activities, "delete_activity", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-activities", deprecated=False, rich_help_panel="API commands")
def get_activities(
    ctx: typer.Context,
    album_id: str = typer.Option(..., "--album-id", help=""""""),
    asset_id: str | None = typer.Option(None, "--asset-id", help=""""""),
    level: ReactionLevel | None = typer.Option(None, "--level", help=""""""),
    type: ReactionType | None = typer.Option(None, "--type", help=""""""),
    user_id: str | None = typer.Option(None, "--user-id", help=""""""),
) -> None:
    """List all activities

    [link=https://api.immich.app/endpoints/activities/getActivities]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["album_id"] = album_id
    if asset_id is not None:
        kwargs["asset_id"] = asset_id
    if level is not None:
        kwargs["level"] = level
    if type is not None:
        kwargs["type"] = type
    if user_id is not None:
        kwargs["user_id"] = user_id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.activities, "get_activities", ctx, **kwargs)
    print_response(result, ctx)


@app.command(
    "get-activity-statistics", deprecated=False, rich_help_panel="API commands"
)
def get_activity_statistics(
    ctx: typer.Context,
    album_id: str = typer.Option(..., "--album-id", help=""""""),
    asset_id: str | None = typer.Option(None, "--asset-id", help=""""""),
) -> None:
    """Retrieve activity statistics

    [link=https://api.immich.app/endpoints/activities/getActivityStatistics]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["album_id"] = album_id
    if asset_id is not None:
        kwargs["asset_id"] = asset_id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(
        client, client.activities, "get_activity_statistics", ctx, **kwargs
    )
    print_response(result, ctx)



"""Generated CLI commands for People tag (auto-generated, do not edit)."""

from __future__ import annotations

import typer
import json
from typing import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from immich import AsyncClient

from immich.cli.runtime import print_response, run_command, set_nested
from immich.client.generated.models import *

app = typer.Typer(
    help="""A person is a collection of faces, which can be favorited and named. A person can also be merged into another person. People are automatically created via the face recognition job.\n\n[link=https://api.immich.app/endpoints/people]Immich API documentation[/link]"""
)


@app.command("create-person", deprecated=False, rich_help_panel="API commands")
def create_person(
    ctx: typer.Context,
    birth_date: str | None = typer.Option(
        None,
        "--birth-date",
        help="""Person date of birth.
Note: the mobile app cannot currently set the birth date to null.""",
    ),
    color: str | None = typer.Option(None, "--color", help=""""""),
    is_favorite: Literal["true", "false"] | None = typer.Option(
        None, "--is-favorite", help=""""""
    ),
    is_hidden: Literal["true", "false"] | None = typer.Option(
        None, "--is-hidden", help="""Person visibility"""
    ),
    name: str | None = typer.Option(None, "--name", help="""Person name."""),
) -> None:
    """Create a person

    [link=https://api.immich.app/endpoints/people/createPerson]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    if birth_date is not None:
        set_nested(json_data, ["birth_date"], birth_date)
    if color is not None:
        set_nested(json_data, ["color"], color)
    if is_favorite is not None:
        set_nested(json_data, ["is_favorite"], is_favorite.lower() == "true")
    if is_hidden is not None:
        set_nested(json_data, ["is_hidden"], is_hidden.lower() == "true")
    if name is not None:
        set_nested(json_data, ["name"], name)
    person_create_dto = PersonCreateDto.model_validate(json_data)
    kwargs["person_create_dto"] = person_create_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.people, "create_person", ctx, **kwargs)
    print_response(result, ctx)


@app.command("delete-people", deprecated=False, rich_help_panel="API commands")
def delete_people(
    ctx: typer.Context,
    ids: list[str] = typer.Option(..., "--ids", help=""""""),
) -> None:
    """Delete people

    [link=https://api.immich.app/endpoints/people/deletePeople]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    set_nested(json_data, ["ids"], ids)
    bulk_ids_dto = BulkIdsDto.model_validate(json_data)
    kwargs["bulk_ids_dto"] = bulk_ids_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.people, "delete_people", ctx, **kwargs)
    print_response(result, ctx)


@app.command("delete-person", deprecated=False, rich_help_panel="API commands")
def delete_person(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Delete person

    [link=https://api.immich.app/endpoints/people/deletePerson]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.people, "delete_person", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-all-people", deprecated=False, rich_help_panel="API commands")
def get_all_people(
    ctx: typer.Context,
    closest_asset_id: str | None = typer.Option(
        None, "--closest-asset-id", help=""""""
    ),
    closest_person_id: str | None = typer.Option(
        None, "--closest-person-id", help=""""""
    ),
    page: float | None = typer.Option(
        None, "--page", help="""Page number for pagination""", min=1
    ),
    size: float | None = typer.Option(
        None, "--size", help="""Number of items per page""", min=1, max=1000
    ),
    with_hidden: Literal["true", "false"] | None = typer.Option(
        None, "--with-hidden", help=""""""
    ),
) -> None:
    """Get all people

    [link=https://api.immich.app/endpoints/people/getAllPeople]Immich API documentation[/link]
    """
    kwargs = {}
    if closest_asset_id is not None:
        kwargs["closest_asset_id"] = closest_asset_id
    if closest_person_id is not None:
        kwargs["closest_person_id"] = closest_person_id
    if page is not None:
        kwargs["page"] = page
    if size is not None:
        kwargs["size"] = size
    if with_hidden is not None:
        kwargs["with_hidden"] = with_hidden.lower() == "true"
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.people, "get_all_people", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-person", deprecated=False, rich_help_panel="API commands")
def get_person(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Get a person

    [link=https://api.immich.app/endpoints/people/getPerson]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.people, "get_person", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-person-statistics", deprecated=False, rich_help_panel="API commands")
def get_person_statistics(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Get person statistics

    [link=https://api.immich.app/endpoints/people/getPersonStatistics]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.people, "get_person_statistics", ctx, **kwargs)
    print_response(result, ctx)


@app.command("get-person-thumbnail", deprecated=False, rich_help_panel="API commands")
def get_person_thumbnail(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
) -> None:
    """Get person thumbnail

    [link=https://api.immich.app/endpoints/people/getPersonThumbnail]Immich API documentation[/link]
    """
    kwargs = {}
    kwargs["id"] = id
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.people, "get_person_thumbnail", ctx, **kwargs)
    print_response(result, ctx)


@app.command("merge-person", deprecated=False, rich_help_panel="API commands")
def merge_person(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    ids: list[str] = typer.Option(..., "--ids", help=""""""),
) -> None:
    """Merge people

    [link=https://api.immich.app/endpoints/people/mergePerson]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    set_nested(json_data, ["ids"], ids)
    merge_person_dto = MergePersonDto.model_validate(json_data)
    kwargs["merge_person_dto"] = merge_person_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.people, "merge_person", ctx, **kwargs)
    print_response(result, ctx)


@app.command("reassign-faces", deprecated=False, rich_help_panel="API commands")
def reassign_faces(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    data: list[str] = typer.Option(..., "--data", help="""As a JSON string"""),
) -> None:
    """Reassign faces

    [link=https://api.immich.app/endpoints/people/reassignFaces]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    value_data = [json.loads(i) for i in data]
    set_nested(json_data, ["data"], value_data)
    asset_face_update_dto = AssetFaceUpdateDto.model_validate(json_data)
    kwargs["asset_face_update_dto"] = asset_face_update_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.people, "reassign_faces", ctx, **kwargs)
    print_response(result, ctx)


@app.command("update-people", deprecated=False, rich_help_panel="API commands")
def update_people(
    ctx: typer.Context,
    people: list[str] = typer.Option(..., "--people", help="""As a JSON string"""),
) -> None:
    """Update people

    [link=https://api.immich.app/endpoints/people/updatePeople]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    value_people = [json.loads(i) for i in people]
    set_nested(json_data, ["people"], value_people)
    people_update_dto = PeopleUpdateDto.model_validate(json_data)
    kwargs["people_update_dto"] = people_update_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.people, "update_people", ctx, **kwargs)
    print_response(result, ctx)


@app.command("update-person", deprecated=False, rich_help_panel="API commands")
def update_person(
    ctx: typer.Context,
    id: str = typer.Argument(..., help=""""""),
    birth_date: str | None = typer.Option(
        None,
        "--birth-date",
        help="""Person date of birth.
Note: the mobile app cannot currently set the birth date to null.""",
    ),
    color: str | None = typer.Option(None, "--color", help=""""""),
    feature_face_asset_id: str | None = typer.Option(
        None,
        "--feature-face-asset-id",
        help="""Asset is used to get the feature face thumbnail.""",
    ),
    is_favorite: Literal["true", "false"] | None = typer.Option(
        None, "--is-favorite", help=""""""
    ),
    is_hidden: Literal["true", "false"] | None = typer.Option(
        None, "--is-hidden", help="""Person visibility"""
    ),
    name: str | None = typer.Option(None, "--name", help="""Person name."""),
) -> None:
    """Update person

    [link=https://api.immich.app/endpoints/people/updatePerson]Immich API documentation[/link]
    """
    kwargs = {}
    json_data = {}
    kwargs["id"] = id
    if birth_date is not None:
        set_nested(json_data, ["birth_date"], birth_date)
    if color is not None:
        set_nested(json_data, ["color"], color)
    if feature_face_asset_id is not None:
        set_nested(json_data, ["feature_face_asset_id"], feature_face_asset_id)
    if is_favorite is not None:
        set_nested(json_data, ["is_favorite"], is_favorite.lower() == "true")
    if is_hidden is not None:
        set_nested(json_data, ["is_hidden"], is_hidden.lower() == "true")
    if name is not None:
        set_nested(json_data, ["name"], name)
    person_update_dto = PersonUpdateDto.model_validate(json_data)
    kwargs["person_update_dto"] = person_update_dto
    client: "AsyncClient" = ctx.obj["client"]
    result = run_command(client, client.people, "update_person", ctx, **kwargs)
    print_response(result, ctx)
